
from .score import *
from .middle_prepare import *
from .multi_element_veri import rmse_skill_seaborn,score_skill_seaborn
from .middel_high import middle_of_score
from .middle_surface import middle_of_score_surface


import matplotlib.pyplot as plt
import math
import meteva
import numpy as np


def bootstrap(data, B, c, func):
    """
    计算bootstrap置信区间
    :param data: array 保存样本数据
    :param B: 抽样次数 通常B>=1000
    :param c: 置信水平
    :param func: 样本估计量
    :return: bootstrap置信区间上下限
    """
    array = np.array(data)
    n = len(array)
    sample_result_arr = []
    for i in range(B):
        index_arr = np.random.randint(0, n, size=n)
        data_sample = array[index_arr]
        sample_result = func(data_sample)
        sample_result_arr.append(sample_result)
    a = 1 - c
    k1 = int(B * a / 2)
    k2 = int(B * (1 - a / 2))
    auc_sample_arr_sorted = sorted(sample_result_arr)
    lower = auc_sample_arr_sorted[k1]
    higher = auc_sample_arr_sorted[k2]

    return lower, higher


def plot_confidence(result_tdt ,B = 10000 ,c = 0.95
                    ,ncol = 1 ,title_list = None ,vmax = None ,vmin = None ,vmax_delta = None,
                    sparsify_xticks = 1, sup_fontsize = 12 ,xlabel ="时效(单位:hour)" ,ylabel = "Score" ,
                    ylabel_delta = "Score Delta" ,width = 10
                    ,height = 6,
                    dpi=300 ,legend_loc =  "upper center" ,wspace = None ,hspace = None,save_path = None,show = False):

    fig = plt.figure(figsize=(width, height), dpi=dpi)
    spasify = sparsify_xticks

    if isinstance(result_tdt ,list):
        result_tdt_list = result_tdt
    else:
        result_tdt_list = [result_tdt]

    member_list = meteva.base.get_stadata_names(result_tdt_list[0])


    nplot = len(result_tdt_list)
    h_title = sup_fontsize * 0.025
    h_xlabel = sup_fontsize * 0.025
    w_ylabel =  sup_fontsize * 0.05
    w_right =  0.1


    nrow = int(math.ceil(nplot / ncol))
    w_plot = (width -w_right - ncol * w_ylabel) / ncol
    h_plot = (height - nrow * (h_xlabel + h_title)) / nrow


    if wspace is None:
        wspace = w_ylabel
    if hspace is None:
        hspace = h_title + h_xlabel

    for p in range(nplot):

        result1 = meteva.base.sele_by_para(result_tdt_list[p], drop_IV=True)
        result_list, dtime_list = meteva.base.group(result1, g="dtime")
        ndtime = len(result_list)
        x = np.arange(ndtime)

        confi = np.zeros(ndtime)
        score_ctrl = np.zeros(ndtime)
        score_test = np.zeros(ndtime)

        for i in range(len(result_list)):
            result1 = result_list[i]
            score_ctrl[i] = np.mean(result1.iloc[:, -2].values)
            score_test[i] = np.mean(result1.iloc[:, -1].values)
            delta = result1.iloc[:, -1].values - result1.iloc[:, -2].values
            lower, higher = bootstrap(delta, B, c, np.nanmean)
            confi[i] = (higher - lower) / 2

        dscore = score_test - score_ctrl

        vmax0 = np.max(np.array([score_ctrl, score_test]))
        vmin0 = np.min(np.array([score_ctrl, score_test]))


        d0 = (vmax0 - vmin0)
        if vmax is None:
            vmax1 = vmax0 + d0 * 0.3
        else:
            vmax1 = vmax

        if vmin is None:
            vmin1 = vmin0 - d0 * 0.1
        else:
            vmin1 = vmin



        pi = p % ncol
        pj = int(p / ncol)
        rect1 = [(w_ylabel + pi * (wspace + w_plot)) / width,
                 (h_xlabel + (nrow - 1 - pj) * (hspace + h_plot) + 0.4 * h_plot) / height,
                 w_plot / width, 0.6 * h_plot / height]

        score_ctrl[score_ctrl == meteva.base.IV] = 0
        score_test[score_test == meteva.base.IV] = 0
        ax1 = plt.axes(rect1)
        plt.bar(x - 0.2, score_ctrl, width=0.4, label=member_list[0], color="k")
        plt.bar(x + 0.2, score_test, width=0.4, label=member_list[1], color="r")
        plt.legend(fontsize=sup_fontsize * 0.8, ncol=2, loc=legend_loc)
        plt.ylim(vmin1, vmax1)
        plt.ylabel(ylabel, fontsize=sup_fontsize * 0.8)
        plt.xlim(-0.5, ndtime - 0.5)
        if title_list is not None:
            plt.title(title_list[p])

        rect2 = [(w_ylabel + pi * (wspace + w_plot)) / width,
                 (h_xlabel + (nrow - 1 - pj) * (hspace + h_plot)) / height,
                 w_plot / width, 0.4 * h_plot / height]
        ax2 = plt.axes(rect2)
        plt.bar(x, -confi, width=0.4, color="w", edgecolor="k")
        plt.bar(x, confi, width=0.4, color="w", edgecolor="k")
        plt.plot(x, dscore, color="r")
        plt.axhline(y=0, color="k", linewidth=0.5)

        vmax1 = np.max(np.array([confi, np.abs(dscore)]))
        if vmax_delta is None:
            vmax_delta = vmax1 * 1.5

        plt.ylim(-vmax_delta, vmax_delta)
        plt.xlim(-0.5, ndtime - 0.5)
        xticks = x[spasify - 1::spasify]
        xticks_labels = dtime_list[spasify - 1::spasify]
        plt.xticks(xticks, xticks_labels, fontsize=sup_fontsize * 0.7)
        plt.xlabel(xlabel, fontsize=sup_fontsize * 0.8)
        plt.ylabel(ylabel_delta, fontsize=sup_fontsize * 0.8)

    if save_path is None:
        show = True
    else:
        meteva.base.tool.path_tools.creat_path(save_path)
        plt.savefig(save_path,bbox_inches='tight')
        print("检验结果已以图片形式保存至" + save_path)
    if show:
        plt.show()
    plt.close()
    return None

def score_compare(sta_ob_and_fos0,method,grade_list = None,compare = ">=",s = None,B = 1000 ,c = 0.95
                    ,ncol = 1 ,title_list = None ,vmax = None ,vmin = None ,vmax_delta = None,
                    sparsify_xticks = 1, sup_fontsize = 12 ,xlabel =None ,ylabel = "Score" ,
                    ylabel_delta = "Score Delta" ,width = 10
                    ,height = 6,
                    dpi=300 ,legend_loc =  "upper center" ,wspace = None ,
                         hspace = None,save_path = None,show = False):
    if grade_list is not None:
        result_tdt,_ = meteva.product.score_tdt(sta_ob_and_fos0,method,s = s,plot=None,grade_list= grade_list,compare = compare)
    else:
        result_tdt, _ = meteva.product.score_tdt(sta_ob_and_fos0, method, s=s, plot=None)

    plot_confidence(result_tdt,B = B,c = c,ncol = ncol ,title_list=title_list,vmax = vmax,vmin = vmin,vmax_delta=vmax_delta,
                    sparsify_xticks=sparsify_xticks,sup_fontsize=sup_fontsize,xlabel=xlabel,ylabel= ylabel,
                    ylabel_delta=ylabel_delta,width=width,height=height,
                    dpi = dpi,legend_loc=legend_loc,wspace=wspace,hspace=hspace,save_path=save_path,show=show)


def score_confidence(sta_ob_and_fos0,method,grade_list = None,compare = ">=",s = None,B = 1000 ,c = 0.95,
                   plot = "bar",**kwargs):

    sta_sele = meteva.base.sele_by_dict(sta_ob_and_fos0,s = s)
    if grade_list is not  None:
        result_tdt_list,_ = meteva.product.score_tdt(sta_sele,method,plot=None,grade_list= grade_list,compare = compare)
        result_dt, dtime_list = meteva.product.score(sta_sele, method, g="dtime",
                                                     plot=None,grade_list= grade_list,compare = compare)
        member_list = meteva.base.get_stadata_names(result_tdt_list[0])
        ngrade = len(grade_list)
        ndtime = len(dtime_list)
        nmember = len(member_list)

        name_list_dict = {
            "dtime": dtime_list,
            "member":member_list,
            "grade": grade_list,
        }
        lower = np.zeros((ndtime, nmember,ngrade))
        higher = np.zeros((ndtime, nmember,ngrade))

        for g in  range(len(grade_list)):
            result_tdt = result_tdt_list[g]
            for i in range(ndtime):
                result1 = meteva.base.sele_by_para(result_tdt, dtime=dtime_list[i],drop_IV=True)
                for k in range(len(member_list)):
                    member = member_list[k]
                    score = result1[member].values
                    lower1, higher1 = bootstrap(score, B, c, np.nanmean)
                    lower[i, k,g] = lower1
                    higher[i, k,g] = higher1
        score = result_dt

    else:
        result_tdt, _ = meteva.product.score_tdt(sta_sele, method,  plot=None)
        result_dt,dtime_list = meteva.product.score(sta_sele, method,g = "dtime",  plot=None)


        member_list = meteva.base.get_stadata_names(result_tdt)
        ndtime = len(dtime_list)
        nmember = len(member_list)
        lower = np.zeros((ndtime,nmember))
        higher = np.zeros((ndtime, nmember))
        for i in range(ndtime):
            result1 = meteva.base.sele_by_para(result_tdt,dtime = dtime_list[i])
            for k in range(len(member_list)):
                member = member_list[k]
                score = result1[member].values

                lower1, higher1 = bootstrap(score, B, c, np.nanmean)
                lower[i,k] =lower1
                higher[i,k] = higher1

        name_list_dict = {
            "dtime":dtime_list,
            "member": member_list,
        }
        score = result_dt

    delta = (higher - lower)/2
    lower = score - delta
    higher = score + delta
    if plot =="bar":
        meteva.base.bar(score,name_list_dict,axis="dtime",lower = lower,higher = higher, **kwargs)
    elif plot =="line":
        meteva.base.plot(score, name_list_dict,axis="dtime",lower = lower,higher = higher, **kwargs)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Apr 26 23:36:09 2023

"""
#import pyreadr
import numpy as np
import pandas as pd
import sys
import meteva

sys.path.append(r'/Users/tangbuxing/Desktop/tra_test-SAL/ZhuMetic/lib')
# from lib import im,distob,solutionset


def metrV(ob, fo, thresholds, lam1=0.5, lam2=0.5,show=False):
    out = {}

    if (isinstance(ob, np.ndarray) is not True) or (isinstance(fo, np.ndarray) is not True):
        raise Exception("metrV.default: invalid x and/or fo argument.")

    q = len(thresholds)
    out["thresholds"] = thresholds
    OvsM1 = np.zeros((q, 3))


    for threshold in range(q):
        M1im = meteva.method.space.zhumetic.im(fo)
        Oim = meteva.method.space.zhumetic.im(ob)
        Oim['v'] = Oim['v'] >= thresholds[threshold]
        M1im['v'] = M1im['v'] >= thresholds[threshold]
        # Ix = solutionset.solutionset(Oim['v'] >= thresholds[threshold])
        # Im1 = solutionset.solutionset(M1im['v'] >= thresholds[threshold])
        Ix = meteva.method.space.zhumetic.solutionset(Oim)
        Im1 = meteva.method.space.zhumetic.solutionset(M1im)

        # out["OvsM1"][threshold,1] = np.sqrt(np.sum(np.sum((Ix["m"] - Im1["m"])**2, axis = 1)))
        a1 = np.logical_xor(Ix["m"], Im1["m"])  # 逻辑运算
        b1 = np.power(a1, 2)
        c1 = np.sum(b1, axis=0)
        d1 = np.sum(c1)
        OvsM1[threshold, 0] = np.sqrt(d1)
        # out["OvsM1"][threshold,2] = distob(Ix['m'], Im1['m'], distfun = distfun)
        OvsM1[threshold, 1] = meteva.method.space.zhumetic.distob(X=Ix, Y=Im1)
        OvsM1[threshold, 2] = lam1 * OvsM1[threshold, 0] + lam2 * OvsM1[threshold, 1]
        if show:
            print("O vs M1 distOV for thresholds: ", thresholds[threshold,], " = ", OvsM1[threshold,], "\n")



        if show:
            print("M1 vs M2 distOV for thresholds: ", thresholds[threshold,], " = ", OvsM1[threshold,], "\n")
        #out["OvsM1"] = pd.DataFrame(out["OvsM1"], columns=["distOV", "distob", "metrV"])

    out["distOV"] = OvsM1[:,0].tolist()
    out["distDV"] = OvsM1[:, 1].tolist()
    out["metrV"] = OvsM1[:,2].tolist()

    return out


if __name__ == '__main__':

    A = np.zeros((10,12))
    B = np.zeros((10,12))
    B2 = np.zeros((10,12))
    A[1,2] = 3
    B[3,6] = 400
    B2[9,7] = 17
    thresholds = np.array([0.1, 3.1, 500])

    x = A
    fo = B
    thresholds = thresholds
    fo2 = B2
    lam1 = 0.5
    distfun = "distmapfun"
    lam2 = 0.5
    a = None
    show = False
    test_metrV = metrV(ob = A, fo = B, thresholds = np.array([0.1, 3.1, 500]))
    print(test_metrV)


    # data1 = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/ZhuMetic/data/geom000.rda')
    # data2 = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/ZhuMetic/data/geom001.rda')
    # # testobj = mem.space.make_spatialVx(grd_ob=geom000, grd_fo=geom001, loc)
    # look_metrV = metrV_default(x=data1['geom000'].values, fo=data2['geom001'].values, thresholds=np.array([0.1, 3.1]))
    #










import meteva
import datetime
import os

def acc_climate_pre(input_dir,output_path):
    time1 = datetime.datetime(2020,1,1,0)
    grd_list = []
    for i in range(732):
        path = input_dir + r"\\"+str(i+1)+".grib"
        grd = meteva.base.read_griddata_from_grib(path)
        time2 = time1 + datetime.timedelta(hours=12*i)
        meteva.base.set_griddata_coords(grd,gtime=[time2])
        grd_list.append(grd)
    grd_all = meteva.base.concat(grd_list)
    meteva.base.write_griddata_to_nc(grd_all,save_path=output_path,effectiveNum=6)


if __name__ == "__main__":
    #input_dir = r"H:\test_data\input\mem\era-dailyclim\t700_mean"
    #output_path = r"H:\test_data\input\mem\era-dailyclim\t700_mean.nc"
    #acc_climate_pre(input_dir,output_path)

    #meteva.base.path_tools.get_path_list_in_dir()
    path = r"H:\test_data\input\mem\era-dailyclim\nc\z500hPa_mean.nc"
    grd1 = meteva.base.read_griddata_from_nc(path)
    print(grd1)
    # path2 = r"H:\test_data\input\mem\era-dailyclim\z500_mean.nc"
    # grd2 = meteva.base.read_griddata_from_nc(path2)
    # import numpy as np
    # #print(grd1)
    # # print(grd2)
    # print(np.max(np.abs(grd1.values - grd2.values)))
    # # path = r"H:\test_data\input\mem\era-dailyclim\nc\1.grib"
    # # grd = meteva.base.read_griddata_from_grib(path)
    # # print(grd)
    meteva.base.cmaps.rain_24h


import meteva
import numpy as np
import matplotlib.pyplot as plt
import math
from matplotlib.colors import BoundaryNorm
from matplotlib.collections import LineCollection
import matplotlib.animation as animation
import pandas as pd
import copy


def add_map(ax,add_county_line = False,add_worldmap = True,title = None,sup_fontsize = 12,linewidth = [0.3,0.3,0.2],color = ["k","k","k"]):

    if meteva.base.customized_basemap_list is None:
        if add_worldmap:
            meteva.base.tool.plot_tools.add_china_map_2basemap(ax, name="world", edgecolor='k', lw=0.3, encoding='gbk', grid0=None)  # "世界地图"
            meteva.base.tool.plot_tools.add_china_map_2basemap(ax, name="world360", edgecolor='k', lw=0.3, encoding='gbk',
                                                               grid0=None)  # "世界地图"
        meteva.base.tool.plot_tools.add_china_map_2basemap(ax, name="nation", edgecolor='k', lw=0.3,encoding = 'gbk')  #"国界"
        meteva.base.tool.plot_tools.add_china_map_2basemap(ax, edgecolor='k', lw=0.3,encoding = 'gbk')  #"省界"
        if add_county_line:
            meteva.base.tool.plot_tools.add_china_map_2basemap(ax, name="county", edgecolor='k', lw=0.2, encoding='gbk')  # "省界"
        ax.set_title(title,fontsize = sup_fontsize* 0.9)
        return ax
    else:
        for i in range(len(meteva.base.customized_basemap_list)):
            shpfile  = meteva.base.customized_basemap_list[i]
            # print(shpfile)
            encoding = 'utf-8'
            try:
                shp1 = meteva.base.tool.plot_tools.readshapefile(shpfile, default_encoding=encoding)
                lines = meteva.base.tool.plot_tools.LineCollection(shp1, antialiaseds=(1,), zorder=100)
                lines.set_color(color[i])
                lines.set_linewidth(linewidth[i])
                lines.set_label('_nolabel_')
                ax.add_collection(lines)
            except:
                encoding = "gbk"
                shp1 = meteva.base.tool.plot_tools.readshapefile(shpfile, default_encoding=encoding)
                lines = meteva.base.tool.plot_tools.LineCollection(shp1, antialiaseds=(1,), zorder=100)
                lines.set_color(color[i])
                lines.set_linewidth(linewidth[i])
                lines.set_label('_nolabel_')
                ax.add_collection(lines)


def creat_axs(nplot,map_extend,ncol = None,height  = None,width = None,dpi = 300,sup_title = None,sup_fontsize = 12,
              add_county_line = False,add_worldmap = True,add_minmap = None,title_list = None,add_index = None,wspace = None,hspace = None,grid = True,
              xticks_inter = None,yticks_inter = None,linewidth = [0.3,0.3,0.2],color = ["k","k","k"],keep_ticks = 1,
              width_colorbar = 0.5):


    ax_index = []
    if add_index is not None:
        if add_index is True:
            ax_index = ["(a)","(b)","(c)","(d)","(e)","(f)","(g)","(h)","(i)","(j)",
                        "(k)", "(l)", "(m)", "(n)", "(o)", "(p)", "(q)", "(r)", "(s)", "(t)",
                        "(u)", "(v)", "(w)", "(x)", "(y)", "(z)"]
        else:
            ax_index = add_index


    if isinstance(map_extend, list):
        slon = map_extend[0]
        elon = map_extend[1]
        slat = map_extend[2]
        elat = map_extend[3]
        rlon = elon - slon
        rlat = elat - slat
    else:
        slon = map_extend.slon
        slat = map_extend.slat
        elon = map_extend.elon
        elat = map_extend.elat
        rlon = map_extend.elon - map_extend.slon
        rlat = map_extend.elat - map_extend.slat


    if sup_title is None:
        sup_height_title = 0
    else:
        sup_height_title = sup_fontsize * 0.12

    height_title = sup_fontsize * 0.1
    if hspace is None:
        if keep_ticks==1:
            height_hspace = sup_fontsize * 0.025
        else:
            height_hspace = sup_fontsize * 0.005
    else:
        height_hspace = hspace

    if wspace is None:
        width_wspace = height_hspace*2
    else:
        width_wspace = wspace


    if keep_ticks==1:
        width_left_yticks = sup_fontsize * 0.2
        height_bottem_xticsk = sup_fontsize * 0.05
    else:
        width_left_yticks = 0
        height_bottem_xticsk =0

    if ncol is None:
        match_list = []
        for i in range(nplot, 0, -1):
            ncol = i
            nrow = int(math.ceil(nplot / ncol))
            rate = ncol * rlon / (nrow * rlat)
            if rate < 2 and rate > 9 / 16:
                match_list.append([i, ncol * nrow - nplot])
        if len(match_list) == 0:
            ncol = nplot
        else:
            match_array = np.array(match_list)
            min_index = np.argmin(match_array[:, 1])
            ncol = match_array[min_index, 0]
    nrow = int(math.ceil(nplot / ncol))

    if width is None and height is None:
        width = 8

    if width is None:
        height_all_plot = height - height_title - height_bottem_xticsk - (nrow - 1) * height_hspace + sup_height_title
        height_map = height_all_plot / nrow
        width_map = height_map * rlon / rlat
        width_all_plot = width_map * ncol + (ncol - 1) * width_wspace
        width = width_all_plot + width_colorbar + width_left_yticks
    else:
        width_all_plot = width - width_colorbar - width_left_yticks - (ncol - 1) * width_wspace
        width_map = width_all_plot / ncol
        height_map = width_map * rlat / rlon
        height_all_plot = height_map * nrow + (nrow - 1) * height_hspace
        height = height_all_plot + height_title + height_bottem_xticsk + sup_height_title

    vmax = elon
    vmin = slon
    if xticks_inter is None:
        r = rlon
        if r <= 0.1:
            inte = 0.05
        elif r <= 0.5:
            inte = 0.1
        elif r <= 1:
            inte = 0.2
        elif r <= 5 and r > 1:
            inte = 1
        elif r <= 10 and r > 5:
            inte = 2
        elif r < 20 and r >= 10:
            inte = 4
        elif r <= 30 and r >= 20:
            inte = 5
        elif r < 180:
            inte = 10
        else:
            inte = 20
    else:
        inte = xticks_inter

    vmin = inte * (math.ceil(vmin / inte))
    vmax = inte * ((int)(vmax / inte)+0.5)

    xticks = np.arange(vmin, vmax, inte)
    xticks_label = []
    xticks_label_None = []
    for x in range(len(xticks)):
        v1 = xticks[x]
        if v1>=0 and v1 <=180:
            if abs(v1 - int(v1))<1e-7:
                xticks_label.append(str(int(round(v1, 6)))+ "°E")
            else:
                xticks_label.append(str(round(v1, 6))+ "°E")
        else:
            if v1<0:
                v2 = -v1
            else:
                v2 = 360 - v1
            if abs(v1 - int(v1))<1e-7:
                xticks_label.append(str(int(round(v2, 6)))+ "°W")
            else:
                xticks_label.append(str(round(v2, 6))+ "°W")



        xticks_label_None.append("")

    # if xticks[-1] > 0:
    #     xticks_label[-1] = "   " + xticks_label[-1] + "°E"
    # else:
    #     xticks_label[-1] = "   " + xticks_label[-1] + "°W"

    vmax = elat
    vmin = slat
    if yticks_inter is None:
        r = rlat
        if r <= 0.05:
            inte = 0.01
        elif r <= 0.3:
            inte = 0.05
        elif r <= 1:
            inte = 0.1
        elif r <= 5 and r > 1:
            inte = 1
        elif r <= 10 and r > 5:
            inte = 2
        elif r < 20 and r >= 10:
            inte = 4
        elif r <= 30 and r >= 20:
            inte = 5
        else:
            inte = 10
    else:
        inter = yticks_inter

    vmin = inte * (math.ceil(vmin / inte))
    vmax = inte * ((int)(vmax / inte)+0.5)
    yticks = np.arange(vmin, vmax, inte)
    yticks_label = []
    yticks_label_None = []
    for y in range(len(yticks)):
        v1 = yticks[y]
        if abs(v1 - int(v1))<1e-7:
            v1 = int(round(v1, 6))
        else:
            v1 = round(v1, 6)

        if yticks[y] >= 0:
            yticks_label.append(str(v1) + "°N")
        else:
            yticks_label.append(str(v1) + "°S")
        yticks_label_None.append("")

    fig = plt.figure(figsize=(width, height), dpi=dpi)
    y_sup_title = (height_bottem_xticsk  + nrow * height_map +(nrow-1) * height_hspace) / height
    if sup_title is not None:
        plt.suptitle(sup_title, x = width_left_yticks / width,y = y_sup_title,fontsize=sup_fontsize,horizontalalignment ="left",
                     verticalalignment  ="bottom")
    ax_list = []
    min_ax_list = []
    for p in range(nplot):
        pi = p % ncol
        pj = int(p / ncol)

        rect1 = [(width_left_yticks + pi * (width_map + width_wspace)) / width,
                 (height_bottem_xticsk + (nrow - 1 - pj) * (height_map + height_hspace)) / height,
                 width_map / width,
                 height_map / height]
        ax = plt.axes(rect1)
        ax.set_xlim((slon, elon))
        ax.set_ylim((slat, elat))

        if keep_ticks>=0:
            if p >= nplot - ncol or keep_ticks:
                # 最底行画横坐标
                ax.set_xticks(xticks)
                ax.set_xticklabels(xticks_label, fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
            else:
                ax.set_xticklabels("", fontsize=sup_fontsize * 0.9) #, family='Times New Roman')


            if pi ==0 or keep_ticks:
                # 最左侧画纵坐标
                ax.set_yticks(yticks)
                ax.set_yticklabels(yticks_label, fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
            else:
                ax.set_yticklabels("", fontsize=sup_fontsize * 0.9) #, family='Times New Roman')
        else:
            ax.set_xticks([])
            ax.set_yticks([])
        if grid: plt.grid(linestyle = "--",linewidth = 0.5)
        if title_list is None:
            sub_title = None
        else:
            sub_title = title_list[p]
        add_map(ax,add_county_line=add_county_line,add_worldmap=add_worldmap,sup_fontsize=sup_fontsize,title=sub_title,
                linewidth = linewidth,color =color)

        if len(ax_index)>=1:
            ix = slon + 0.02*(elon - slon) * 5 / width_map
            iy = elat - 0.035*(elat - slat) * 5 / height_map
            plt.text(ix, iy, ax_index[p], bbox=dict(fc='white', ec='white',pad = 0),fontsize = sup_fontsize,zorder=100)
        ax_list.append(ax)


        if slon< 75 and elon >130 and elat >50 and slat >3 and slat <25:
            if add_minmap is None:
                print("返回结果中自动添加了南海小地图，请注意返回结果中包含了两个绘图框列表")
                add_minmap = "left"

        if add_minmap is not None:
            if add_minmap == "left" or add_minmap=="right":
                minmap_lon_lat = [105, 123, 0, 20]
                minmap_height_rate = 0.27
                height_bigmap = rect1[3]
                height_minmap = height_bigmap * minmap_height_rate
                width_minmap = height_minmap * (minmap_lon_lat[1] - minmap_lon_lat[0]) * height / (
                            minmap_lon_lat[3] - minmap_lon_lat[2])/width

                width_between_two_map =  height_bigmap *0.01
                sy_minmap = width_between_two_map + rect1[1]
                if add_minmap =="left":
                    sx_minmap = rect1[0] + width_between_two_map
                else:
                    sx_minmap = rect1[0] + rect1[2] - width_minmap - width_between_two_map
                rect_min = [sx_minmap,sy_minmap,width_minmap,height_minmap]
                ax_min = plt.axes(rect_min)
                plt.xticks([])
                plt.yticks([])
                ax_min.set_xlim((minmap_lon_lat[0], minmap_lon_lat[1]))
                ax_min.set_ylim((minmap_lon_lat[2], minmap_lon_lat[3]))
                ax_min.spines["top"].set_linewidth(0.3)
                ax_min.spines["bottom"].set_linewidth(0.3)
                ax_min.spines["right"].set_linewidth(0.3)
                ax_min.spines["left"].set_linewidth(0.3)
                add_map(ax_min, add_worldmap =True)  # "国界"
                min_ax_list.append(ax_min)
    if len(min_ax_list)==0:
        return ax_list
    else:
        return ax_list,min_ax_list



def add_contourf(ax,grd,cmap ="rainbow",clevs= None,add_colorbar = True,cut_colorbar = True,title = None,title_fontsize = 8,clip = None,
                 extend = None,colorbar_location = None,alpha=1,title_loc = "center"):
    slon = ax.transLimits._boxin.x0
    elon = ax.transLimits._boxin.x1
    slat = ax.transLimits._boxin.y0
    elat = ax.transLimits._boxin.y1
    if grd is None:return
    grid0 = meteva.base.get_grid_of_data(grd)
    grid1 = meteva.base.grid([slon,elon,grid0.dlon],[slat,elat,grid0.dlat])
    grd1 = meteva.base.interp_gg_linear(grd,grid1,outer_value=np.nan)

    x = grd1['lon'].values
    y = grd1['lat'].values
    vmax = np.nanmax(grd1.values)
    vmin = np.nanmin(grd1.values)
    if extend is None: extend = "neither"
    cmap1,clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap,clevs=clevs,vmin=vmin,vmax = vmax,cut_colorbar = cut_colorbar,extend =extend)
    # if extend == "neither":
    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N - 1)
    # elif extend == "both":
    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N + 1)
    # else:
    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
    norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
    im = ax.contourf(x, y, np.squeeze(grd1.values), levels=clevs1, cmap=cmap1,norm = norm,alpha=alpha,extend = extend)
    fig = plt.gcf()
    width = fig.bbox.width/fig.dpi
    height = fig.bbox.height/fig.dpi

    ax.set_title(title,fontsize =title_fontsize,loc = title_loc)
    if add_colorbar:
        if colorbar_location is None:
            location = [ax.bbox.x1 / fig.dpi / width + 0.005, ax.bbox.y0 / fig.dpi / height, 0.01,
                        ax.bbox.height / fig.dpi / height]
        else:
            location = colorbar_location
        colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
        plt.colorbar(im,cax= colorbar_position)

    if clip is not None:
        try:
            meteva.base.tool.maskout.shp2clip_by_shpfile(im,ax,clip)
        except:
            if isinstance(clip, str): clip = [clip]
            if isinstance(clip[0], str):
                meteva.base.tool.maskout.shp2clip_by_region_name(im, ax, clip)
            else:
                meteva.base.tool.maskout.shp2clip_by_lines(im, ax, clip)

    return im

def add_contour(ax,grd,color='k', linewidth = 1,label_fontsize = 5,clevs = None,title = None,title_fontsize = 8,clip = None):


    slon = ax.transLimits._boxin.x0
    elon = ax.transLimits._boxin.x1
    slat = ax.transLimits._boxin.y0
    elat = ax.transLimits._boxin.y1

    grid0 = meteva.base.get_grid_of_data(grd)
    grid1 = meteva.base.grid([slon,elon,grid0.dlon],[slat,elat,grid0.dlat])
    grd1 = meteva.base.interp_gg_linear(grd,grid1,outer_value=np.nan)

    x = grd1['lon'].values
    y = grd1['lat'].values
    vmax = np.nanmax(grd1.values)
    vmin = np.nanmin(grd1.values)


    if clevs is None:
        dif = (vmax - vmin) / 10.0
        inte = math.pow(10, math.floor(math.log10(dif)));
        #用基本间隔，将最大最小值除于间隔后小数点部分去除，最后把间隔也整数化
        r = dif / inte
        if r < 3 and r >= 1.5:
            inte = inte * 2
        elif r < 4.5 and r >= 3:
            inte = inte * 4
        elif r < 5.5 and r >= 4.5:
            inte = inte * 5
        elif r < 7 and r >= 5.5:
            inte = inte * 6
        elif r >= 7:
            inte = inte * 8
        vmin = inte * (math.floor(vmin / inte))
        vmax = inte * (math.ceil(vmax / inte) + 0.5)
        clevs = np.arange(vmin, vmax, inte)
    mm = max(abs(clevs[0]),abs(clevs[-1]))
    if mm!=0:
        mmi = math.ceil(math.log10(mm))
        if mmi >=2:
            f1 = mmi
            f2 = 0
        elif mmi == 1:
            f1 = 2
            f2 = 1
        else:
            mmi = math.floor(math.log10(mm))
            f1 = 1+abs(mmi)
            f2 = f1
    else:
        f1 = 1
        f2 = 0
    fmt = "%"+str(f1)+"."+str(f2)+"f"
    ax.set_title(title,fontsize =title_fontsize)
    im = ax.contour(x, y, np.squeeze(grd1.values),levels = clevs,colors = color,linewidths = linewidth)
    ax.clabel(im, inline=1, fontsize=label_fontsize,fmt=fmt)
    if clip is not None:
        try:
            meteva.base.tool.maskout.shp2clip_by_shpfile(im,ax,clip)
        except:
            if isinstance(clip, str): clip = [clip]
            if isinstance(clip[0], str):
                meteva.base.tool.maskout.shp2clip_by_region_name(im, ax, clip)
            else:
                meteva.base.tool.maskout.shp2clip_by_lines(im, ax, clip)

    return im



def add_mesh(ax,grd,cmap ="rainbow",clevs= None,add_colorbar = True,title = None,title_fontsize = 8,extend = None,colorbar_location = None):


    slon = ax.transLimits._boxin.x0
    elon = ax.transLimits._boxin.x1
    slat = ax.transLimits._boxin.y0
    elat = ax.transLimits._boxin.y1

    grid0 = meteva.base.get_grid_of_data(grd)
    grid1 = meteva.base.grid([slon,elon,grid0.dlon],[slat,elat,grid0.dlat])
    grd1 = meteva.base.interp_gg_linear(grd,grid1,outer_value=np.nan)

    x = grd1['lon'].values
    y = grd1['lat'].values
    vmax = np.nanmax(grd1.values)
    vmin = np.nanmin(grd1.values)
    if extend is None: extend="neither"
    cmap1,clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap,clevs=clevs,vmin=vmin,vmax = vmax,extend=extend)
    # if extend is None:
    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N - 1)
    # elif extend == "both":
    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
    #     #norm = BoundaryNorm(clevs1, ncolors=cmap1.N + 1, extend=extend)
    # else:
    #     #norm = BoundaryNorm(clevs1, ncolors=cmap1.N,extend = extend)
    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
    norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
    im = ax.pcolormesh(x, y, np.squeeze(grd1.values), cmap=cmap1, norm=norm)
    #im = ax.contourf(x, y, np.squeeze(grd.values), levels=clevs1, cmap=cmap1,norm = norm)

    fig = plt.gcf()

    #left_low = (width +0.1 - 0.8) / width

    ax.set_title(title,fontsize =title_fontsize)
    if add_colorbar:
        if colorbar_location is None:
            width = fig.bbox.width / fig.dpi
            height = fig.bbox.height / fig.dpi
            location = [ax.bbox.x1 / fig.dpi / width + 0.005, ax.bbox.y0 / fig.dpi / height, 0.01,
                        ax.bbox.height / fig.dpi / height]
        else:
            location = colorbar_location

        colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
        plt.colorbar(im,cax= colorbar_position,extend = extend)
    return im

def add_barbs(ax,wind,color = "k",skip = None,title = None,title_fontsize = 8,length = None):

    fig = plt.gcf()
    width = fig.bbox.width / fig.dpi

    if isinstance(wind, pd.DataFrame):
        if length is None:
            length = ax.bbox.width / fig.dpi
        X = wind["lon"].values
        Y = wind["lat"].values
        u = wind.iloc[:,-2].values
        v = wind.iloc[:,-1].values
        ax.set_title(title, fontsize=title_fontsize)
        im = ax.barbs(X, Y, u, v,
                      sizes=dict(emptybarb=0.01, spacing=0.23, height=0.5, width=0.25), color=color,
                      barb_increments=dict(half=2, full=4, flag=20), length=length, linewidth=length * length * 0.03)
    else:
        slon = ax.transLimits._boxin.x0
        elon = ax.transLimits._boxin.x1
        slat = ax.transLimits._boxin.y0
        elat = ax.transLimits._boxin.y1

        grid0 = meteva.base.get_grid_of_data(wind)
        grid1 = meteva.base.grid([slon,elon,grid0.dlon],[slat,elat,grid0.dlat])
        wind1 = meteva.base.interp_gg_linear(wind,grid1,outer_value=np.nan)

        x = wind1['lon'].values
        y = wind1['lat'].values
        X, Y = np.meshgrid(x, y)
        u = np.squeeze(wind1.values[0,...])
        v = np.squeeze(wind1.values[1,...])


        if skip is  not None:
            if length is None:
                #length = math.sqrt((width-1) * skip / x.size) * 5
                length = math.sqrt(ax.bbox.width*skip/ fig.dpi)*1.0
        else:
            if length is None:
                length = ax.bbox.width/fig.dpi
            skip =int((length /10)**2 * x.size / (width - 1))+1
        ax.set_title(title,fontsize =title_fontsize)
        im = ax.barbs(X[::skip,::skip],Y[::skip,::skip] , u[::skip,::skip], v[::skip,::skip],
                 sizes=dict(emptybarb=0.01, spacing=0.23, height=0.5,width = 0.25),color = color,
                 barb_increments=dict(half=2, full=4, flag=20),length = length,linewidth = length * length  * 0.03)


    return im


def add_streamplot(ax,wind,color = "k",title = None,title_fontsize = 8,density= 1,linewidth = 1):

    slon = ax.transLimits._boxin.x0
    elon = ax.transLimits._boxin.x1
    slat = ax.transLimits._boxin.y0
    elat = ax.transLimits._boxin.y1

    grid0 = meteva.base.get_grid_of_data(wind)
    grid1 = meteva.base.grid([slon,elon,grid0.dlon],[slat,elat,grid0.dlat])
    wind1 = meteva.base.interp_gg_linear(wind,grid1,outer_value=np.nan)

    x = wind1['lon'].values
    y = wind1['lat'].values
    X, Y = np.meshgrid(x, y)
    u = np.squeeze(wind1.values[0,...])
    v = np.squeeze(wind1.values[1,...])

    fig = plt.gcf()
    width = fig.bbox.width/fig.dpi

    ax.set_title(title,fontsize =title_fontsize)
    im = ax.streamplot(X,Y , u, v, color = color,density = density,linewidth=linewidth,arrowstyle = "->")

    return im

def add_meteor(ax,grd_wind,skip = 5,meteor_length = 1,meteor_width = 1,cmap = "viridis"):
    L = 20
    grid0 = meteva.base.get_grid_of_data(grd_wind)
    grid1 = meteva.base.grid([grid0.slon, grid0.elon, grid0.dlon * skip], [grid0.slat, grid0.elat, grid0.dlat * skip])
    grd_station = meteva.base.grid_data(grid1)
    station = meteva.base.trans_grd_to_sta(grd_station)
    N = len(station.index)
    point_list  = [copy.deepcopy(station[["lon","lat"]].values)]
    #积分出轨迹
    for i in range(L):
        sta_uv = meteva.base.interp_gs_linear(grd_wind,station)

        station.iloc[:,4] += sta_uv.iloc[:,6] * meteor_length * 0.01
        station.iloc[:, 5] += sta_uv.iloc[:, 7] * meteor_length * 0.01
        point_list.append(copy.deepcopy(station[["lon", "lat"]].values))

    points = np.array(point_list)
    points_s = points[:-1,:,:].reshape(-1,2)
    points_e = points[1:,:,:].reshape(-1,2)
    cc = np.repeat(np.arange(1,L),N).T.flatten()
    segs = np.stack([points_s, points_e], axis=1)
    lc = LineCollection(segs, cmap=cmap, capstyle='round')
    lc.set_array(cc)
    lc.set_linewidth(cc*meteor_width*0.05)
    image = ax.add_collection(lc)

    return image


def add_animation(ax,grd_wind,skip = 5,meteor_length = 1,meteor_width = 1,cmap = "viridis"):
    grid0 = meteva.base.get_grid_of_data(grd_wind)
    grid1 = meteva.base.grid([grid0.slon, grid0.elon, grid0.dlon * skip], [grid0.slat, grid0.elat, grid0.dlat * skip])
    grd_station = meteva.base.grid_data(grid1)
    station = meteva.base.trans_grd_to_sta(grd_station)
    N = len(station.index)
    point_list = [copy.deepcopy(station[["lon", "lat"]].values)]
    L_all = 50
    # 积分出轨迹
    for i in range(L_all):
        sta_uv = meteva.base.interp_gs_linear(grd_wind, station)
        station.iloc[:, 4] += sta_uv.iloc[:, 6] * meteor_length * 0.02
        station.iloc[:, 5] += sta_uv.iloc[:, 7] * meteor_length * 0.02
        point_list.append(copy.deepcopy(station[["lon", "lat"]].values))
    points = np.array(point_list)
    points[-1,:,:] = np.nan

    L_show = 10

    for i in range(N):
        split_index = np.random.randint(1, L_all -1)
        part1 = points[:split_index,i,:]
        part2 = points[split_index:,i,:]
        new_arr = np.concatenate((part2, part1))
        points[:,i,:] = new_arr

    points = np.concatenate((points,points[0:L_show+1,:,:]))


    arr = points[:L_show+1 , :, :]
    nan_rows = np.isnan(arr).any(axis=(0, 2))     # 找到包含 nan 的行的索引
    new_arr = arr[:,~nan_rows,:]     # 删除包含 nan 的行
    #print(new_arr[:,0,0])
    points_s = new_arr[:-1 , :, :].reshape(-1, 2)
    points_e = new_arr[ 1:, :, :].reshape(-1, 2)
    segs = np.stack([points_s, points_e], axis=1)
    lc = LineCollection(segs, cmap=cmap, capstyle='round')
    ax.add_collection(lc)


    def update(frame):
        arr = points[frame:frame+L_show + 1, :, :]
        nan_rows = np.isnan(arr).any(axis=(0, 2))  # 找到包含 nan 的行的索引
        new_arr = arr[:, ~nan_rows, :]  # 删除包含 nan 的行
        points_s = new_arr[:-1, :, :].reshape(-1, 2)
        points_e = new_arr[1:, :, :].reshape(-1, 2)

        segs = np.stack([points_s, points_e], axis=1)
        lc.set_segments(segs)
        N1 = new_arr.shape[1]
        cc = np.repeat(np.arange(1, L_show), N1).T.flatten()
        lc.set_array(cc)
        lc.set_linewidth(cc * meteor_width * 0.1)
        return  [lc]

    fig = ax.figure
    ani = animation.FuncAnimation(fig, update, frames=L_all, interval=50, blit=True, repeat=True)
    return ani


def add_scatter(ax,sta0,cmap = "rainbow",clevs = None,point_size = None,fix_size = True,threshold = 2,min_spot_value = 0,mean_value = 2,
                add_colorbar = True,alpha = None,title = None,title_fontsize = 8,extend = None,colorbar_location = None):

    slon = ax.transLimits._boxin.x0
    elon = ax.transLimits._boxin.x1
    slat = ax.transLimits._boxin.y0
    elat = ax.transLimits._boxin.y1
    rlon = elon - slon
    rlat = elat - slat
    sta = sta0

    sta_without_iv = meteva.base.sele.not_IV(sta)
    data_names = meteva.base.get_stadata_names(sta_without_iv)
    sta_without_iv = sta_without_iv.sort_values(by=data_names[-1], ascending=True)

    values = sta_without_iv.iloc[:, -1].values

    vmax_v = np.max(sta_without_iv.iloc[:,-1].values)
    vmin_v = np.min(sta_without_iv.iloc[:,-1].values)
    if extend is None: extend="neither"
    cmap1, clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap, clevs=clevs, vmin=vmin_v, vmax=vmax_v,extend = extend)
    # if extend is None:
    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N - 1)
    # elif extend == "both":
    #     #norm = BoundaryNorm(clevs1, ncolors=cmap1.N + 2, extend=extend)
    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
    # else:
    #     #norm = BoundaryNorm(clevs1, ncolors=cmap1.N+1, extend=extend)
    #     norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
    norm = BoundaryNorm(clevs1, ncolors=cmap1.N)
    fig = plt.gcf()
    map_width = ax.bbox.width/fig.dpi

    if point_size is None:
        sta_id1 = sta0.drop_duplicates(['id'])
        sta_dis = meteva.base.sta_dis_ensemble_near_by_sta(sta_id1, nearNum=2)
        dis_values = sta_dis["data1"].values
        dis_values.sort()
        dis1 = dis_values[int(len(dis_values) * 0.02) + 1]/1.2
        point_size = (map_width * dis1 / rlon) ** 2
        # point_size = 100 * map_area / len(sta.index)
        # print("**************")
        # print(point_size)
        if (point_size > 30): point_size = 30
        if (point_size < 0.5):
            point_size = 0.5
            if alpha is None:
                alpha = 0.5
        # point_size *=3
        #left_low = (width + 0.1 - right_plots_width) / width


    x = sta_without_iv.loc[:, "lon"].values
    y = sta_without_iv.loc[:, "lat"].values
    colors = values
    if isinstance(fix_size, bool):
        if fix_size:
            im = ax.scatter(x, y, c=colors, cmap=cmap1, norm=norm, s=point_size,edgecolors = "face",alpha = alpha)
        else:
            area = point_size * np.abs(values - min_spot_value) / mean_value
            if (threshold is not None):
                area[np.abs(values - min_spot_value) < threshold] *= 0.1
            im = ax.scatter(x, y, c=colors, cmap=cmap1, norm=norm, s=area,edgecolors  = 'face',alpha = alpha)

    else:
        im = ax.scatter(x, y, c=colors, cmap=cmap1, norm=norm, s=fix_size,edgecolors  = 'face',alpha = alpha)
    ax.set_title(title,fontsize =title_fontsize)


    if add_colorbar:
        if colorbar_location is None:
            width = fig.bbox.width / fig.dpi
            height = fig.bbox.height / fig.dpi
            location = [ax.bbox.x1 / fig.dpi / width + 0.005, ax.bbox.y0 / fig.dpi / height, 0.01,
                        ax.bbox.height / fig.dpi / height]
        else:
            location = colorbar_location

        colorbar_position = fig.add_axes(location)  # 位置[左,下,宽,高]
        plt.colorbar(im, cax=colorbar_position,extend = extend)
    return im

def add_scatter_text(ax,sta0,color = "k",cmap = None,clevs = None,tag = 2,
               alpha = 1,font_size = 10,title = None,title_fontsize = 8):
    x0 = ax.transLimits._boxin.x0
    x1 = ax.transLimits._boxin.x1
    y0 = ax.transLimits._boxin.y0
    y1 = ax.transLimits._boxin.y1
    sta = meteva.base.sele_by_para(sta0,lon=[x0,x1],lat=[y0,y1])
    fig = plt.gcf()
    if isinstance(sta.iloc[0,-1],str):
        nsta = len(sta.index)
        for i in range(nsta):
            x = sta.iloc[i, 4]
            y = sta.iloc[i, 5]
            v = sta.iloc[i, -1]
            ax.text(x, y, v, ha="center", va="center", fontsize=font_size, color=color, clip_on=True, alpha=alpha,
                        zorder=1000)

    else:

        sta_without_iv = meteva.base.sele.not_IV(sta)
        data_names = meteva.base.get_stadata_names(sta_without_iv)
        sta_without_iv = sta_without_iv.sort_values(by=data_names[-1], ascending=True)

        vmax_v = np.max(sta_without_iv.iloc[:,-1].values)
        vmin_v = np.min(sta_without_iv.iloc[:,-1].values)
        clevs1 = None
        cmap1 = None
        if cmap is not None:
            cmap1, clevs1 = meteva.base.tool.color_tools.def_cmap_clevs(cmap=cmap, clevs=clevs, vmin=vmin_v, vmax=vmax_v)

        fmt_tag = "%." + str(tag) + "f"

        if cmap1 is None:
            nsta = len(sta_without_iv.index)
            for i in range(nsta):
                x = sta_without_iv.iloc[i,4]
                y = sta_without_iv.iloc[i,5]
                v = sta_without_iv.iloc[i,-1]
                ax.text(x, y, fmt_tag % v, ha="center", va="center",fontsize=font_size,color = color, clip_on=True,alpha = alpha, zorder=1000)

        else:
            x = sta_without_iv.iloc[:, 4].values
            y = sta_without_iv.iloc[:, 5].values
            v = sta_without_iv.iloc[:, -1].values
            for k in range(len(clevs1)-1):
                index = np.where((v>=clevs1[k])&(v<clevs1[k+1]))
                if len(index[0])>0:
                    x1 = x[index]
                    y1 = y[index]
                    v1 = v[index]
                    color = cmap1(k)
                    for j in range(x1.size):
                        if isinstance(v1[j],str):
                            ax.text(x1[j], y1[j], v1[j], ha="center", va="center", fontsize=font_size, c=color, clip_on=True,alpha = alpha, zorder=1000)
                        else:
                            ax.text(x1[j], y1[j], fmt_tag % v1[j], ha="center", va="center", fontsize=font_size,  c=color, clip_on=True,alpha = alpha, zorder=1000)

    ax.set_title(title,fontsize =title_fontsize)
    return

def add_closed_line(ax,graphy,color = "k",linewidth=2,fontsize = 10,title = None,title_fontsize = 8):
    if graphy is None:
        return
    x0 = ax.transLimits._boxin.x0
    x1 = ax.transLimits._boxin.x1
    y0 = ax.transLimits._boxin.y0
    y1 = ax.transLimits._boxin.y1

    contours =graphy["closed_contours"]
    ncontour = len(contours["cn_label"])


    for n in range(ncontour):
        line_array = contours["cn_xyz"][n][:,0:2]
        ax.plot(line_array[:,0], line_array[:,1], color=color, linewidth=linewidth)
        value = contours["cn_label"][n]
        i_s = -1
        line_dict = None
        while i_s < line_array.shape[0]-1:
            i_s += 1
            x = line_array[i_s,0]
            y = line_array[i_s,1]
            if x >=x0 and x<= x1 and y >=y0 and y <= y1:
                if i_s == 0:
                    x_1 = line_array[1,0]
                    y_1 = line_array[1,1]
                    if x_1 == x:
                        rotation = 90
                    else:
                        rotation = math.atan((y_1 - y)/(x_1 - x)) * 180 / math.pi
                    ax.text(x, y, value, ha="center", va="center", fontsize=fontsize, color=color,
                            rotation = rotation,bbox ={"facecolor":"w","pad":0,"linewidth":0})
                    break
                else:
                    if line_dict is None:
                        line_dict = {}
                        line_dict["x"] = []
                        line_dict["y"] = []
                    else:
                        line_dict["x"].append(x)
                        line_dict["y"].append(y)
            else:
                if line_dict is not None:
                    np = len(line_dict["x"])
                    if np >5:
                        np_half = int(np/2)
                        x = line_dict["x"][np_half]
                        y = line_dict["y"][np_half]
                        x_1 =  line_dict["x"][np_half+1]
                        y_1 = line_dict["y"][np_half+1]
                        if x_1 == x:
                            rotation = 90
                        else:
                            rotation = math.atan((y_1 - y) / (x_1 - x)) * 180 / math.pi
                        ax.text(x, y, value, ha="center", va="center", fontsize=fontsize, color=color,
                                 rotation=rotation,bbox ={"facecolor":"w","pad":0,"linewidth":0})
                        line_dict = None
    ax.set_title(title,fontsize =title_fontsize)
    return



def add_cyclone_trace(ax,sta_cyclone_trace,size = 0.3,linewidth = 1,title = None,title_fontsize = 8):

    np = len(sta_cyclone_trace.index)
    for i in range(np - 1):
        lon = [sta_cyclone_trace.iloc[i,4],sta_cyclone_trace.iloc[i+1,4]]
        lat = [sta_cyclone_trace.iloc[i,5],sta_cyclone_trace.iloc[i+1,5]]
        speed = sta_cyclone_trace.iloc[i+1,-1]


        if speed >= 10.8 and speed <17.2:
            color = "k"
        elif speed >=17.2 and speed <24.5:
            color = "b"
        elif speed >= 24.5 and speed < 32.7:
            color = "y"
        elif speed >= 32.7 and speed < 41.5:
            color = "orange"
        elif speed >=41.5 and speed < 51:
            color = "r"
        else:
            color = "m"
        ax.plot(lon, lat, c = color, linewidth=linewidth, zorder=29)
        time_ob = sta_cyclone_trace.iloc[i,1]

        if (time_ob.hour - 2) %12 ==0:
            r_o = size
            lon_list = []
            lat_list = []
            for a in range(0,360,5):
                if a> 70 and a<135:
                    lon1 = lon[0] + r_o *2 * math.cos(70 * math.pi/180)
                    lat1 = lat[0] + r_o *2 * math.sin(70 * math.pi/180)
                elif a >250 and a < 315:
                    lon1 = lon[0] + r_o * 2 * math.cos(250 * math.pi / 180)
                    lat1 = lat[0] + r_o * 2 * math.sin(250 * math.pi / 180)
                else:
                    theta = math.pi * a / 180
                    lon1 = lon[0] + r_o * math.cos(theta)
                    lat1 = lat[0] + r_o * math.sin(theta)
                lon_list.append(lon1)
                lat_list.append(lat1)
            ax.fill(lon_list,lat_list,c = color,zorder = 30,linewidth=0)
            r_o = size * 0.3
            lon_list = []
            lat_list = []
            for a in range(0,360,5):
                theta = math.pi * a / 180
                lon1 = lon[0] + r_o * math.cos(theta)
                lat1 = lat[0] + r_o * math.sin(theta)
                lon_list.append(lon1)
                lat_list.append(lat1)
            ax.fill(lon_list,lat_list,c = "white",zorder = 30,linewidth=0)
    ax.set_title(title,fontsize =title_fontsize)
    return

def add_solid_lines(ax,graphy,color = "r",linewidth = None,title = None,title_fontsize = 8):
    if graphy is None:
        return
    slon = ax.transLimits._boxin.x0
    elon = ax.transLimits._boxin.x1
    slat = ax.transLimits._boxin.y0
    elat = ax.transLimits._boxin.y1

    rlon = elon - slon
    rlat = elat - slat
    fig = plt.gcf()
    map_width = ax.bbox.width / fig.dpi

    if linewidth is None:
        linewidth = rlon *  0.015/ map_width

    line_list = []
    if isinstance(graphy,dict):
        features = graphy["features"]
        for value in features.values():
            line = value["axes"]
            line_list.append(line["point"])
    else:
        line_list = graphy

    for line in line_list:
        point = np.array(line)
        if point.size>2:
            ax.plot(point[:, 0], point[:, 1],color = color, linewidth=linewidth)

    ax.set_xlim(slon,elon)
    ax.set_ylim(slat,elat)
    ax.set_title(title,fontsize =title_fontsize)
    return

def add_double_solid_lines(ax,graphy,color = None,linewidth = 1,line_inter = 2,title = None,title_fontsize = 8):
    if graphy is None:
        return

    slon = ax.transLimits._boxin.x0
    elon = ax.transLimits._boxin.x1
    slat = ax.transLimits._boxin.y0
    elat = ax.transLimits._boxin.y1

    rlon = elon - slon
    rlat = elat - slat
    fig = plt.gcf()
    map_width = ax.bbox.width / fig.dpi

    line_inter = rlon *  line_inter * 0.01/ map_width

    line_list = []
    if isinstance(graphy,dict):
        features = graphy["features"]
        for value in features.values():
            line = value["axes"]
            line_type = "c"
            if "line_type" in line.keys():
                line_type = line["line_type"]
            if color is None:
                if line_type == "c":
                    color = "b"
                else:
                    color = "r"
            line_list.append(line["point"])
    else:
        line_list = graphy

    for line in line_list:
        point = np.array(line)
        dp = np.zeros(point.shape)
        dp[:-1,:] = point[1:,:] - point[:-1,:]
        dp[-1,:] = dp[-2,:]
        length = (dp[:,0]**2 + dp[:,1] **2)**0.5
        dx = -dp[:,1] * line_inter/ length
        dy = dp[:,0] * line_inter/length

        ax.plot(point[:,0] - dx ,point[:,1] - dy,color,linewidth = linewidth)
        ax.plot(point[:, 0] + dx, point[:, 1] + dy, color,linewidth =  linewidth)

    ax.set_xlim(slon,elon)
    ax.set_ylim(slat,elat)
    ax.set_title(title,fontsize =title_fontsize)
    return



def add_curved_arrows(ax,graphy,color = "red",linewidth = None,head_width = 1,head_length = 1,title = None,title_fontsize = 8):
    if graphy is None:
        return
    slon = ax.transLimits._boxin.x0
    elon = ax.transLimits._boxin.x1
    slat = ax.transLimits._boxin.y0
    elat = ax.transLimits._boxin.y1

    rlon = elon - slon
    rlat = elat - slat
    fig = plt.gcf()
    map_width = ax.bbox.width / fig.dpi
    if linewidth is None:
        linewidth = rlon *  0.15/ map_width

    line_list = []
    if isinstance(graphy, dict):
        features = graphy["features"]
        for value in features.values():
            line = value["axes"]
            line_list.append(line["point"])
    else:
        line_list = graphy

    for line in line_list:
        point = np.array(line)
        npoint = len(line)
        if npoint>4:
            ns = npoint -1
            dx = 0
            dy = 0
            while ns >0:
                ns -= 1
                dx = point[npoint-1,0] - point[ns,0]
                dy = point[npoint-1,1] - point[ns,1]
                dis = (dx**2 + dy**2)**0.5
                if dis > 0.3:
                    break

            ax.arrow(point[ns-1,0],point[ns-1,1],dx*0.01,dy*0.01,head_width=head_width,head_length = head_length,
                     fc = color,ec = color)
            ax.plot(point[:ns, 0], point[:ns, 1], color, linewidth=linewidth)

    ax.set_xlim(slon,elon)
    ax.set_ylim(slat,elat)
    ax.set_title(title,fontsize =title_fontsize)
    return

def add_cold_fronts(ax,graphy,color = "b",linewidth = None,
                   triangle_step = 10,triangle_size = 1,title = None,title_fontsize = 8):
    if graphy is None:
        return
    slon = ax.transLimits._boxin.x0
    elon = ax.transLimits._boxin.x1
    slat = ax.transLimits._boxin.y0
    elat = ax.transLimits._boxin.y1

    rlon = elon - slon
    rlat = elat - slat
    fig = plt.gcf()
    map_width = ax.bbox.width / fig.dpi

    if linewidth is None:
        linewidth = rlon *  0.015/ map_width

    line_list = []
    if isinstance(graphy, dict):
        features = graphy["features"]
        for value in features.values():
            line = value["axes"]
            line_list.append(line["point"])
    else:
        line_list = graphy

    for line in line_list:
        point = np.array(line)
        if point.size>4:
            ax.plot(point[:, 0], point[:, 1],color, linewidth=linewidth)

        npoint = len(point)

        start = int(triangle_step/2)
        for i in range(start,npoint,triangle_step):
            point0 = point[i,:]
            for j in range(i,npoint-3):
                point1_ = point[j]
                dis1 = np.sqrt(np.sum(np.power(point1_ - point0,2)))
                if dis1 > triangle_size:
                    rate = triangle_size/dis1
                    delta = rate*(point1_ - point0)
                    point1 = point0 + delta
                    point_mid = (point0+point1)/2
                    delta_rotate90 = meteva.base.tool.math_tools.rotate_vector(delta/2,90)
                    point2 = point_mid + delta_rotate90
                    vertices = np.array([point0, point1,point2])
                    triangle = plt.Polygon(vertices, closed=True, fill=True, color=color)
                    ax.add_patch(triangle)
                    break

    ax.set_xlim(slon,elon)
    ax.set_ylim(slat,elat)
    ax.set_title(title,fontsize =title_fontsize)
    return




def add_warm_fronts(ax,graphy,color = "r",linewidth = None,
                   semicircle_step = 10,semicircle_size = 1,title = None,title_fontsize = 8):
    if graphy is None:
        return
    slon = ax.transLimits._boxin.x0
    elon = ax.transLimits._boxin.x1
    slat = ax.transLimits._boxin.y0
    elat = ax.transLimits._boxin.y1

    rlon = elon - slon
    rlat = elat - slat
    fig = plt.gcf()
    map_width = ax.bbox.width / fig.dpi

    if linewidth is None:
        linewidth = rlon *  0.015/ map_width

    line_list = []
    if isinstance(graphy, dict):
        features = graphy["features"]
        for value in features.values():
            line = value["axes"]
            line_list.append(line["point"])
    else:
        line_list = graphy

    for line in line_list:
        point = np.array(line)
        if point.size>4:
            ax.plot(point[:, 0], point[:, 1],color, linewidth=linewidth)

        npoint = len(point)

        start = int(semicircle_step/2)
        for i in range(start,npoint,semicircle_step):
            point0 = point[i,:]
            for j in range(i,npoint-3):
                point1_ = point[j]
                dis1 = np.sqrt(np.sum(np.power(point1_ - point0,2)))
                if dis1 > semicircle_size:
                    rate = semicircle_size/dis1
                    delta = rate*(point1_ - point0)
                    point1 = point0 + delta
                    point_mid = (point0+point1)/2

                    # 计算从 point2 到半圆起点的角度（假设 point2 在 x 轴正方向为 0 度）
                    angle_to_point2 = np.arctan2(delta[1], delta[0])

                    # 半圆的角度范围（从 point2 开始，逆时针到相反方向 point2 的位置）
                    start_angle = angle_to_point2
                    end_angle = start_angle + np.pi

                    # 生成角度数据
                    theta = np.linspace(start_angle, end_angle, 100)

                    # 计算半圆上点的坐标
                    x = point_mid[0] + semicircle_size * np.cos(theta)/2
                    y = point_mid[1] + semicircle_size * np.sin(theta)/2

                    vertices = np.array([x,y]).T
                    semicircle = plt.Polygon(vertices, closed=True, fill=True, color=color)
                    ax.add_patch(semicircle)

                    break

    ax.set_xlim(slon,elon)
    ax.set_ylim(slat,elat)
    ax.set_title(title,fontsize =title_fontsize)
    return


import numpy as np
import pandas as pd
import math
import meteva
#import pyreadr


def imomenter_matrix_numpy(x, loc=None):
    out = {}

    if loc is None:
        x_dim = x.shape[0]
        y_dim = x.shape[1]
        range0 = np.tile(np.arange(1, x_dim + 1), y_dim)
        range1 = (np.arange(1, y_dim + 1)).repeat(x_dim)
        loc = np.stack((range0, range1), axis=-1)

    def mij(x, s=None, i=0, j=0):
        x = np.array(x)
        if s is None:
            x_dim = x.shape[0]
            y_dim = x.shape[1]
            range0 = np.tile(np.arange(1, x_dim + 1), y_dim)
            range1 = (np.arange(1, y_dim + 1)).repeat(x_dim)
            s = np.stack((range0, range1), axis=-1)
        s_0 = np.zeros(s.shape)
        s_0[:, 0] = np.power(s[:, 0], i)
        s_0[:, 1] = np.power(s[:, 1], j)
        s = np.prod(s_0, axis=1)
        result = np.sum(s * np.reshape(x, -1, 'F'))
        return result

    M00 = mij(x, s=loc)
    M10 = mij(x, s=loc, i=1, j=0)
    M01 = mij(x, s=loc, i=0, j=1)
    M11 = mij(x, s=loc, i=1, j=1)
    M20 = mij(x, s=loc, i=2, j=0)
    M02 = mij(x, s=loc, i=0, j=2)

    xbar = M10 / M00
    ybar = M01 / M00
    cen = np.array([xbar, ybar])
    #names_cen = pd.DataFrame(cen, columns=["x", "y"])
    mu11 = M11 / M00 - xbar * ybar
    mu20 = M20 / M00 - xbar ** 2
    mu02 = M02 / M00 - ybar ** 2
    theta = 0.5 * math.atan2(2 * mu11, mu20 - mu02)
    raw = np.array([[M00, M10, M01, M11, M20, M02]])
    raw_moments = pd.DataFrame(raw, columns=["M00", "M10", "M01", "M11", "M20", "M02"])
    cov = np.array([[mu20, mu11], [mu11, mu02]])

    out = {'volume': M00, 'centroid': cen, 'OrientationAngle': theta,
           'raw_moments': raw, 'cov': cov, 'class': "imomented"}

    return out


def imomenter_matrix_one(look, label, ob_or_fo):
    grid0 = look["grid"]
    value = None
    if ob_or_fo == "ob":
        tmp = look['grd_ob_features'][label]
        value = look["grd_ob"].values.squeeze()
    elif ob_or_fo == "fo":
        tmp = look['grd_fo_features'][label]
        value = look["grd_fo"].values.squeeze()
    else:
        if label in look["grd_features"].keys():
            tmp = look['grd_features'][label]
        else:
            return None

    x = np.zeros((grid0.nlat, grid0.nlon))
    x[tmp] = 1
    x = x * value

    result_0 = imomenter_matrix_numpy(x.T)  #不带单位的结果
    result = {}
    result["volume"] = result_0["volume"] * grid0.dlon * grid0.dlat
    result["centroid"] = {"x":result_0["centroid"][0] * grid0.dlon + grid0.slon,
                            "y":result_0["centroid"][1] * grid0.dlat + grid0.slat}
    result["OrientationAngle"] = result_0["OrientationAngle"]
    result["cov"] = {
        "cov_xx":result_0["cov"][0,0] *  grid0.dlon * grid0.dlon,
        "cov_yy": result_0["cov"][1, 1] * grid0.dlat * grid0.dlat,
        "cov_xy": result_0["cov"][1, 0] * grid0.dlon* grid0.dlat
    }

    return result

def imomenter_matrix(look_merge):
    nmatch = look_merge["match_count"]
    out = {}
    out["time"] = meteva.base.all_type_time_to_str(look_merge["grd_fo"]["time"].values[0])
    out["dtime"] = int(look_merge["grd_fo"]["dtime"].values[0])
    out["member"] = look_merge["grd_fo"]["member"].values[0]
    out["match_count"] = nmatch


    label_list_matched = look_merge["label_list_matched"]
    out["label_list_matched"] = label_list_matched
    out["unmatched"] = look_merge["unmatched"]

    for i in label_list_matched:
        f_axis_ob = imomenter_matrix_one(look_merge, i, "ob")
        f_axis_fo = imomenter_matrix_one(look_merge, i, "fo")
        out1 = {}
        out1["imomenter_matrix"] = {"ob":f_axis_ob,"fo":f_axis_fo}
        out[i] = out1

    miss_labels = look_merge["unmatched"]['ob']
    for i in miss_labels:
        f_axis_ob = imomenter_matrix_one(look_merge, i, "ob")
        if i in out.keys():
            out1 = out[i]
            out1["imomenter_matrix"] = {"ob": f_axis_ob}
        else:
            out1 = {}
            out1["imomenter_matrix"] = {"ob":f_axis_ob}

        out[i] = out1


    false_alarm_labels = look_merge["unmatched"]["fo"]
    for i in false_alarm_labels:
        f_axis_fo = imomenter_matrix_one(look_merge, i, "fo")
        if i in out.keys():
            out1 = out[i]
            out1["feature_axis"] ={"fo": f_axis_fo}
        else:
            out1 = {}
            out1["feature_axis"] = {"fo": f_axis_fo}

        out[i] = out1

    return out


if __name__ == '__main__':

    import meteva.base as meb
    import meteva.method as mem
    # 读取观测和预报数据
    grid1 = meb.grid([100, 120, 0.05], [24, 40, 0.05])
    path_ob = r'H:\test_data\input\mem\mode\ob\rain03\20072611.000.nc'
    path_fo_03 = r'H:\test_data\input\mem\mode\ec\rain03\20072608.003.nc'
    path_fo_27 = r'H:\test_data\input\mem\mode\ec\rain03\20072508.027.nc'
    grd_ob = meb.read_griddata_from_nc(path_ob, grid=grid1, time="2020072611", dtime=0, data_name="OBS")
    grd_fo_03 = meb.read_griddata_from_nc(path_fo_03, grid=grid1, time="2020072608", dtime=3, data_name="ECMWF")
    grd_fo_27 = meb.read_griddata_from_nc(path_fo_27, grid=grid1, time="2020072508", dtime=27, data_name="ECMWF")
    look_ff = mem.mode.feature_finder(grd_ob, grd_fo_03, smooth=5, threshold=5, minsize=5)
    print("*** 目标识别完成 ***\n")
    look_match = mem.mode.centmatch(look_ff)
    look_merge = mem.mode.merge_force(look_match)
    result = imomenter_matrix(look_merge)
    print(result)


import datetime
import os
import json
import traceback
import numpy as np
import pandas as pd
import meteva
import matplotlib.pyplot as plt
import pkg_resources
import xarray as xr


def set_jar_path(path):
    if os.path.exists(path):
        meteva.base.ws_jar_path = path
        meteva.base.height_oy = os.path.split(path)[0] +"/height_oy.nc"
    else:
        print(path + " not exists")

def java_class_func(jar_path, class_name, func_name, jvm_path=None, *args):
    import jpype
    """
    调用jar包中class下的方法
    :return:
    """
    # jar包路径的配置
    # jarpath = os.path.join(os.path.abspath("."), "D:\\hf-0.1.jar")
    jarpath = os.path.join(os.path.abspath(".."), jar_path)
    # 这里指定了jvm
    if jvm_path:
        jvmpath = jvm_path
    else:
        jvmpath = jpype.getDefaultJVMPath()

    try:
        jpype.startJVM(jvmpath, "-ea", "-Djava.class.path=%s" % jarpath)
    except Exception as e:
        pass

    java_class = jpype.JClass(class_name)
    ru_param = ','.join(list(map(lambda x: json.dumps(x), args)))

    # f1 = open(r"d:/para.txt","w",encoding="utf-8")
    # f1.write(ru_param)
    # f1.close()
    res = str(eval("java_class.%s(%s)" % (func_name, ru_param)))
    #jpype.shutdownJVM()
    return res




def high_low_center(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100,grade_interval = 5):
    grid0 = meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()
    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()

    para = {"type":"high_low_center",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,"grade_interval":grade_interval,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon": grid_h.slon, "h_slat": grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data": h_data
            }
    para_json = json.dumps(para)

    file1 = open(r"H:\task\develop\java\sysIdentify\para_json.txt","w")
    file1.write(para_json)
    file1.close()

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None

def vortex(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100):
    grid0 = meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()
    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()
    para = {"type":"vortex",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon": grid_h.slon, "h_slat": grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data": h_data
            }
    para_json = json.dumps(para)


    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None


def anti_vortex(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 500):
    grid0 = meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()
    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()
    para = {"type":"anti_vortex",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon": grid_h.slon, "h_slat": grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data": h_data
            }
    para_json = json.dumps(para)

    # file1 = open(r"H:\task\develop\java\sysIdentify\para_json.txt","w")
    # file1.write(para_json)
    # file1.close()


    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None

def trough(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100):
    grid0 = meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()

    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()

    para = {"type":"trough",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon":grid_h.slon,"h_slat":grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data":h_data}
    para_json = json.dumps(para)

    file1 = open(r"H:\task\develop\java\sysIdentify\para_json.txt","w")
    file1.write(para_json)
    file1.close()

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None


def reverse_trough(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100):
    grid0 = meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()
    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()
    para = {"type":"reverse_trough",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon": grid_h.slon, "h_slat": grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data": h_data
            }
    para_json = json.dumps(para)
    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None

def convergence_line(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()
    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()
    para = {"type":"convergence_line",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon": grid_h.slon, "h_slat": grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data": h_data
            }
    para_json = json.dumps(para)

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None


def shear(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()
    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()
    para = {"type":"shear",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon": grid_h.slon, "h_slat": grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data": h_data
            }
    para_json = json.dumps(para)

    file1 = open(r"H:\task\develop\java\sysIdentify\para_json.txt","w")
    file1.write(para_json)
    file1.close()

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None

def jet(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 100,jet_min_speed = 12,only_south_jet =False):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()
    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()
    para = {"type":"jet",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":resolution,"jet_min_speed":jet_min_speed,"only_south_jet":str(only_south_jet),
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon": grid_h.slon, "h_slat": grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data": h_data
            }
    para_json = json.dumps(para)



    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None


def subtropical_high(grd,output_dir_root,smooth_times = 0,min_size = 500,necessary_height = 5840,sufficient_height = 5880):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()
    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()
    para = {"type":"subtropical_high",
            "smooth_times":smooth_times,"min_size":min_size,"necessary_height":necessary_height,"sufficient_height":sufficient_height,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon": grid_h.slon, "h_slat": grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data": h_data
            }
    para_json = json.dumps(para)

    # file1 = open(r"H:\task\develop\java\sysIdentify\para_json.txt","w")
    # file1.write(para_json)
    # file1.close()

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None


def south_asia_high(grd,output_dir_root,smooth_times = 0,min_size = 800,sn_height = 16680):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()
    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()
    para = {"type":"south_asia_high",
            "smooth_times":smooth_times,"min_size":min_size,"sn_height":sn_height,
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon": grid_h.slon, "h_slat": grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data": h_data
            }
    para_json = json.dumps(para)

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None

def tran_graphy_to_df(graphy):
    '''

    :param graphy:
    :return:
    '''
    if graphy is None: return None,None

    dtime = int(graphy["dtime"])

    time  = meteva.base.all_type_time_to_time64(graphy["time"])

    level = graphy["level"]

    ids = graphy["features"].keys()

    list1 = []
    center = None
    for id in ids:
        feature = graphy["features"][id]
        lon = feature["center"]["lon"]
        lat = feature["center"]["lat"]
        center_value = round(feature["center"]["value"],2)
        if len(feature["region"].keys())>0:
            area = round(feature["region"]["area"],2)
            strength = round(feature["region"]["strength"])
            dict1 = {"level":level,"time":time,"dtime":dtime,"id" :int(id),"lon":lon,"lat":lat,"value":center_value,"area":area,"strength":strength}
            list1.append(dict1)
    if len(list1)>0:
        center = pd.DataFrame(list1)

    list1 = []
    for id in ids:
        points = graphy["features"][id]["axes"]["point"]
        if len(points)>0:
            array = np.array(points)
            lon = array[:,0]
            lat = array[:,1]
            df = pd.DataFrame({"lon":lon,"lat":lat})
            df["id"] = int(id)
            list1.append(df)
    if len(list1)>0:
        df = pd.concat(list1,axis = 0)
        df["level"] = level
        df["time"] = time
        df["dtime"] = dtime
        df["data0"] = 0
        axes = meteva.base.sta_data(df)
        return center,axes
    else:
        return center,None

def cold_front(grd,output_dir_root,smooth_times = 0,min_size = 500):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()
    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()
    para = {"type":"cold_front",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":"low",
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon": grid_h.slon, "h_slat": grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data": h_data
            }
    para_json = json.dumps(para)

    # file1 = open(r"H:\task\develop\java\sysIdentify\para_json.txt","w")
    # file1.write(para_json)
    # file1.close()

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None


def warm_front(grd,output_dir_root,smooth_times = 10,min_size = 300):
    grid0 =  meteva.base.get_grid_of_data(grd)
    data = grd.values.flatten().tolist()
    height_oy = meteva.base.read_griddata_from_nc(meteva.base.height_oy)
    grid_h = meteva.base.get_grid_of_data(height_oy)
    h_data = height_oy.values.flatten().tolist()
    para = {"type":"warm_front",
            "smooth_times":smooth_times,"min_size":min_size,"resolution":"low",
            "level" : int(grid0.levels[0]),"time" : grid0.stime_str[0:10],"dtime" : int(grid0.dtimes[0]),
            "nlon":grid0.nlon,"nlat":grid0.nlat,"startlon":float(grid0.slon),"startlat":float(grid0.slat),"dlon":float(grid0.dlon),"dlat":float(grid0.dlat),
            "data":data,"output_dir_root":output_dir_root,
            "h_nlon": grid_h.nlon, "h_nlat": grid_h.nlat,
            "h_slon": grid_h.slon, "h_slat": grid_h.slat,
            "h_dlon": grid_h.dlon, "h_dlat": grid_h.dlat,
            "h_data": h_data
            }
    para_json = json.dumps(para)

    file1 = open(r"H:\task\develop\java\sysIdentify\para_json.txt","w")
    file1.write(para_json)
    file1.close()

    str_json = java_class_func(meteva.base.ws_jar_path,"Jpype","ws",None,para_json)
    if str_json !="null" and len(str_json)>0:
        graphy = json.loads(str_json)
        return graphy
    else:
        return None


if __name__=="__main__":
    jar_path = r"H:\task\develop\java\sysIdentify\out\artifacts\sysIdentify_jar\sysIdentify.jar"
    set_jar_path(jar_path)
    # grid6 = meteva.base.grid([0, 359.75, 0.25], [-89.75, 89.75, 0.25])
    # #grid6 = meteva.base.grid([150, 230, 0.5], [15, 65, 0.5])
    # t850 = meteva.base.read_griddata_from_micaps4(r"H:\test_data\input\mem\ws\ECMWF\T\850\18010820.000", grid=grid6)
    # t850.values -= meteva.base.K
    # output_dir_root = r"H:\test_data\output\method\ws"
    # graphy_wf = warm_front(t850, output_dir_root, smooth_times=30, min_size=500)

    # axs = meb.creat_axs(1, map_extend=grid6, ncol=1, add_minmap=False, wspace=1)
    # t850_smooth = meb.smooth(t850, 5)
    # meb.add_contourf(axs[0], t850_smooth, cmap=meb.cmaps.temp_2m)
    # meb.add_warm_fronts(axs[0], graphy_wf, linewidth=1)  # 绘制暖锋


    output_dir_root = r"H:\test_data\output\method\ws"
    #
    #path = r"H:\task\link\fengyun\20221230.nc"
    grid6 = meteva.base.grid([0, 359.5, 0.5], [-89.5, 89.5, 0.5])
    #h = meteva.base.read_griddata_from_nc(path,grid = grid6,level=500,dtime = 0)
    #trough(h,output_dir_root,smooth_times=30,min_size=500)
    path = r"H:\task\link\fengyun\ECMWF_D1D_FOR_GLB_U_20250307000000_12_100_50000.MIC"
    wind = meteva.base.read_gridwind_from_micaps11(path,grid = grid6,level=0,dtime=0)
    shear(wind,output_dir_root,min_size= 500)
    print()
    #path = r"\task\link\fengyun\1747909521636.m4.MIC"
    #grd = meteva.base.read_griddata_from_micaps4(path)
    # print(grd)
    #
    # #meteva.base.write_griddata_to_nc(grd,r"H:\task\develop\java\OAL\test_data\hgt\500\2024022420.000.nc")
    #
    #high_low_center(grd,output_dir_root,resolution = "low",smooth_times = 0,min_size = 1,grade_interval = 40)


    # path = r"O:\data\grid\ECMWF_HR\WIND\850\20220326\22032608.000.nc"
    # #
    # # import xarray
    # # wind = xarray.open_dataset(path)
    # # print(wind)
    # wind = meteva.base.read_griddata_from_nc(path)
    # print(wind)


    #save_path = r"H:\test_data\input\mem\ws\ECMWF\WIND\850\22032608.000"
    #wind = meteva.base.read_gridwind_from_micaps11(save_path,level=0,time = datetime.datetime(2022,3,26,8),dtime = 0)

    #meteva.base.write_griddata_to_micaps11(wind,save_path=r"H:\test_data\input\mem\ws\ECMWF\WIND\850\22032608.000",creat_dir=True)

    #graphy_anti_vortex =anti_vortex(wind,output_dir_root,smooth_times = 5)

    # grid6 = meteva.base.grid([150, 230, 0.5], [0, 80, 0.5])
    # t850 = meteva.base.read_griddata_from_micaps4(r"H:\test_data\input\mem\ws\ECMWF\T\850\18010820.000",grid = grid6)
    # graphy_cf = warm_front(t850, output_dir_root, smooth_times=30)

    # # 读取1000hPa位势高度场
    # h1000 = meteva.base.read_griddata_from_micaps4(r"H:\test_data\input\mem\ws\ECMWF\H\1000\22062600.012",
    #                                        level=1000, time="2022062600", dtime=12)
    # # 调用高低压识别算法
    # graphy_hl = meteva.method.weather_system.high_low_center(h1000, output_dir_root, smooth_times=10)
    # print(graphy_hl.keys())  # 返回结果的字典中包含了 type, level,time,dtime 和 features等关键词
    # save_path = r"H:\test_data\input\mem\ws\ECMWF\T\850\a.txt"
    # grd = meteva.base.read_griddata_from_gds_file(save_path)
    # save_path = r"H:\test_data\input\mem\ws\ECMWF\T\850\19071208.000"
    # meteva.base.write_griddata_to_micaps4(grd,save_path)

    #path = r"\\10.28.16.234\data2\AI\ECMWF\tigge\t850_201801.grib"
    # import xarray as xr
    # grd = xr.open_dataset(path)
    # print(grd)
    #grd = meteva.base.read_griddata_from_grib(path, value_name="t", dtime_dim="step", level_type="isobaricInhPa")
    #print(grd)

    # time1 = datetime.datetime(2018,1,1,8)
    # while time1 < datetime.datetime(2018,2,1,8):
    #     time_ut = time1 - datetime.timedelta(hours=8)
    #     #grd1 = meteva.base.sele_by_para(grd,time=[time_ut],dtime = 0)
    #     path = meteva.base.get_path(r"\\10.28.16.234\data2\AI\CRA40\2018\T\850\YYYYMMDDHH.nc",time_ut)
    #     grd1 = meteva.base.read_griddata_from_nc(path)
    #     save_path = meteva.base.get_path(r"H:\test_data\input\mem\ws\ECMWF\T\850\YYMMDDHH.000",time1)
    #
    #     meteva.base.write_griddata_to_micaps4(grd1,save_path=save_path)
    #     time1 += datetime.timedelta(hours=12)
    #


    # time1 = datetime.datetime(2018,1,1,8)
    # while time1 < datetime.datetime(2018,2,1,8):
    #     time_ut = time1 - datetime.timedelta(hours=8)
    #     #grd1 = meteva.base.sele_by_para(grd,time=[time_ut],dtime = 0)
    #     path = meteva.base.get_path(r"\\10.28.16.234\data2\AI\CRA40\2018\MSL\YYYYMMDDHH.nc",time_ut)
    #     grd1 = meteva.base.read_griddata_from_nc(path)
    #     grd1.values /=100
    #     save_path = meteva.base.get_path(r"H:\test_data\input\mem\ws\ECMWF\MSL\YYMMDDHH.000",time1)
    #
    #     meteva.base.write_griddata_to_micaps4(grd1,save_path=save_path,creat_dir=True)
    #     time1 += datetime.timedelta(hours=12)




import math
import os.path

import meteva.base as meb
import datetime
import numpy as np
import matplotlib.pyplot as plt
import meteva
import matplotlib.ticker as ticker
import xarray as xr

def spectrum_Spherical_Harmonic_one_field(grd):
    import pyshtools as sh
    """
    计算标量场的球谐能谱
    :param data: 输入高度场数据（Nx×Ny数组，经度×纬度）
    :param l_max: 最大球谐度数
    :return: 波数l数组，能谱E(l)
    """
    # 1. 创建SHGrid对象（需确保输入数据为规则网格）

    data = grd.values.squeeze()
    if data.shape[0] * 2 -1 != data.shape[1] and data.shape[0] * 2  != data.shape[1]:
        print("data.shape:",data.shape)
        print("进行球谐变换时需保证东西向格点数=南北向格点数×2-1(或0)",
              "number of grid points in the east-west direction should equal to the number of grid points in the north-south direction * 2 - 1(or 0)")
        return
    grid = sh.SHGrid.from_array(data)

    # 2. 执行球谐展开
    coeffs = grid.expand(normalization='ortho', csphase=-1) #, lmax_calc=l_max)

    # 3. 提取球谐系数
    height_coeffs = coeffs.coeffs
    l_max = height_coeffs.shape[1]

    # 4. 计算能谱（各向同性平均）
    E_l = np.zeros(l_max)
    for l in range(l_max):
        # 对每个l的所有m求和
        E_l[l] = 0.5 * np.sum(np.abs(height_coeffs[:, l, :]) ** 2) / (2 * l + 1)

    # 5. 归一化（可选）
    E_l_normalized = E_l / (4 * np.pi)

    return E_l_normalized


def spectrum_Spherical_Harmonic(para,show = None,save_path = None,title = None):
    grid0 = para["grid"]
    begin_time = para["begin_time"]
    end_time = para["end_time"]
    dtime = para["dtime"]
    time_step = para["time_step"]
    time1 = begin_time - datetime.timedelta(hours=dtime)
    members = list(para["fo_data"].keys())

    sp_list_list = []
    while time1 <= end_time:
        time1 += datetime.timedelta(hours=time_step)

        time_ob = time1 + datetime.timedelta(hours=dtime)
        if para["time_type"] != para["ob_data"]["time_type"]:
            if para["time_type"]=="UT":
                time_ob_file = time_ob + datetime.timedelta(hours=8)
            else:
                time_ob_file = time_ob - datetime.timedelta(hours=8)
        else:
            time_ob_file = time_ob

        path_ob = meb.get_path(para["ob_data"]["dir_ob"],time_ob_file)
        print(path_ob)
        if not os.path.exists(path_ob):continue
        grd = para["ob_data"]["read_method"](path_ob,grid = grid0,**para["ob_data"]["read_para"],time = time_ob_file,show = True)
        if grd is None:continue
        if para["ob_data"]["operation"] is not None:
            grd = para["ob_data"]["operation"](grd,**para["ob_data"]["operation_para"])

        sp_list = []
        sp = spectrum_Spherical_Harmonic_one_field(grd)
        sp_list.append(sp)

        for member in members:

            if para["time_type"] != para["fo_data"][member]["time_type"]:
                if para["time_type"] == "UT":
                    time_fo_file = time1 + datetime.timedelta(hours=8)
                else:
                    time_fo_file = time1 - datetime.timedelta(hours=8)
            else:
                time_fo_file = time1

            path_fo = meb.get_path(para["fo_data"][member]["dir_fo"], time_fo_file,dtime)
            print(path_fo)
            if not os.path.exists(path_fo): continue
            grd = para["fo_data"][member]["read_method"](path_fo, grid=grid0, **para["fo_data"][member]["read_para"],time = time_fo_file,dtime = dtime,show = True)
            if grd is None: continue
            if para["fo_data"][member]["operation"] is not None:
                grd = para["fo_data"][member]["operation"](grd, **para["fo_data"][member]["operation_para"])


            sp = spectrum_Spherical_Harmonic_one_field(grd)
            sp_list.append(sp)

        if len(sp_list) == len(para["fo_data"].keys()) +1:
            sp_list_list.append(sp_list)


    sp_array = np.array(sp_list_list)
    sp_array_mean = np.mean(sp_array,axis=0)  #时间维度取平均
    sp_dict = {}
    sp_dict["obs"] = sp_array_mean[0,:]
    for m in range(len(members)):
        member = members[m]
        sp_dict[member] = sp_array_mean[m+1,:]


    if show is not None or save_path is not None:
        ns = 2
        ne = int(grid0.nlon / 4)
        for key in sp_dict.keys():
            sp = sp_dict[key]
            plt.plot(sp[ns:ne], label=key, linewidth=1)

        plt.yscale('log')  # 设置Y轴为对数坐标
        plt.xscale('log')  # 设置Y轴为对数坐标


        plt.xlabel('WaveNumber')

        if title is not None:
            plt.title(title)
        plt.ylabel('energy')
        plt.legend()

        if save_path is not None:
            print(save_path)
            plt.savefig(save_path, dpi=600)
            print("png result has been output to " + save_path)
        if show is not None:
            plt.show()

        plt.close()

    return sp_dict


def kinetic_energy_spectrum(para):
    grid0 = para["grid"]
    begin_time = para["begin_time"]
    end_time = para["end_time"]
    dtime = para["dtime"]
    time_step = para["time_step"]
    time1 = begin_time - datetime.timedelta(hours=dtime)
    members = list(para["fo_data"].keys())

    sp_list_list = []
    sp_list_list_vor = []
    sp_list_list_div = []
    while time1 <= end_time:
        time1 += datetime.timedelta(hours=time_step)

        time_ob = time1 + datetime.timedelta(hours=dtime)
        if para["time_type"] != para["ob_data"]["u"]["time_type"]:
            if para["time_type"] == "UT":
                time_ob_file = time_ob + datetime.timedelta(hours=8)
            else:
                time_ob_file = time_ob - datetime.timedelta(hours=8)
        else:
            time_ob_file = time_ob

        path_ob = meb.get_path(para["ob_data"]["u"]["dir_ob"], time_ob_file)
        print(path_ob)
        if not os.path.exists(path_ob): continue
        grd_u = para["ob_data"]["u"]["read_method"](path_ob, grid=grid0, **para["ob_data"]["u"]["read_para"],
                                                    time=time_ob_file, show=True)
        if grd_u is None: continue
        if para["ob_data"]["u"]["operation"] is not None:
            grd_u = para["ob_data"]["u"]["operation"](grd_u, **para["ob_data"]["u"]["operation_para"])

        path_ob = meb.get_path(para["ob_data"]["v"]["dir_ob"], time_ob_file)
        grd_v = para["ob_data"]["v"]["read_method"](path_ob, grid=grid0, **para["ob_data"]["v"]["read_para"],
                                                    time=time_ob_file, show=True)
        if grd_v is None: continue
        if para["ob_data"]["v"]["operation"] is not None:
            grd_v = para["ob_data"]["v"]["operation"](grd_v, **para["ob_data"]["v"]["operation_para"])

        sp_list = []
        sp_u = spectrum_Spherical_Harmonic_one_field(grd_u)
        sp_v = spectrum_Spherical_Harmonic_one_field(grd_v)
        sp_list.append(sp_u + sp_v)

        sp_list_div = []
        sp_list_vor = []

        if para["need_Rotational_Divergent"]:
            div = meteva.base.uv_to_div(grd_u, grd_v)
            vor = meteva.base.uv_to_vor(grd_u, grd_v)

            sp_vor = spectrum_Spherical_Harmonic_one_field(vor)
            sp_div = spectrum_Spherical_Harmonic_one_field(div)

            sp_list_vor.append(sp_vor)
            sp_list_div.append(sp_div)

        for member in members:

            if para["time_type"] != para["fo_data"][member]["u"]["time_type"]:
                if para["time_type"] == "UT":
                    time_fo_file = time1 + datetime.timedelta(hours=8)
                else:
                    time_fo_file = time1 - datetime.timedelta(hours=8)
            else:
                time_fo_file = time1

            path_fo = meb.get_path(para["fo_data"][member]["u"]["dir_fo"], time_fo_file, dtime)
            print(path_fo)
            if not os.path.exists(path_fo): continue
            grd_u = para["fo_data"][member]["u"]["read_method"](path_fo, grid=grid0,
                                                                **para["fo_data"][member]["u"]["read_para"],
                                                                time=time_fo_file, dtime=dtime, show=True)
            if grd_u is None: continue
            if para["fo_data"][member]["u"]["operation"] is not None:
                grd_u = para["fo_data"][member]["u"]["operation"](grd_u,
                                                                  **para["fo_data"][member]["u"]["operation_para"])
            sp_u = spectrum_Spherical_Harmonic_one_field(grd_u)

            path_fo = meb.get_path(para["fo_data"][member]["v"]["dir_fo"], time_fo_file, dtime)
            grd_v = para["fo_data"][member]["v"]["read_method"](path_fo, grid=grid0,
                                                                **para["fo_data"][member]["v"]["read_para"],
                                                                time=time_fo_file, dtime=dtime, show=True)
            if grd_v is None: continue
            if para["fo_data"][member]["v"]["operation"] is not None:
                grd_v = para["fo_data"][member]["v"]["operation"](grd_v,
                                                                  **para["fo_data"][member]["v"]["operation_para"])
            sp_v = spectrum_Spherical_Harmonic_one_field(grd_v)

            sp_list.append(sp_u + sp_v)

            if para["need_Rotational_Divergent"]:
                div = meteva.base.uv_to_div(grd_u, grd_v)
                vor = meteva.base.uv_to_vor(grd_u, grd_v)

                sp_vor = spectrum_Spherical_Harmonic_one_field(vor)
                sp_div = spectrum_Spherical_Harmonic_one_field(div)

                sp_list_vor.append(sp_vor)
                sp_list_div.append(sp_div)

        if len(sp_list) == len(para["fo_data"].keys()) + 1:
            sp_list_list.append(sp_list)
            if para["need_Rotational_Divergent"]:
                sp_list_list_div.append(sp_list_div)
                sp_list_list_vor.append(sp_list_vor)

    sp_array = np.array(sp_list_list)
    sp_array_mean = np.mean(sp_array, axis=0)  # 时间维度取平均

    if para["need_Rotational_Divergent"]:
        sp_array_div = np.array(sp_list_list_div)
        sp_array_mean_div = np.mean(sp_array_div, axis=0)  # 时间维度取平均
        sp_array_vor = np.array(sp_list_list_vor)
        sp_array_mean_vor = np.mean(sp_array_vor, axis=0)  # 时间维度取平均

        # 归一化处理
        rate = (sp_array_mean_div + sp_array_mean_vor) / sp_array_mean
        sp_array_mean_div /= rate
        sp_array_mean_vor /= rate

        obs_name = para["ob_data"]["data_name"]

        sp_dict = {}
        sp_dict[obs_name] = sp_array_mean[0, :]
        sp_dict[f"{obs_name}_rot"] = sp_array_mean_vor[0, :]
        sp_dict[f"{obs_name}_div"] = sp_array_mean_div[0, :]

        for m in range(len(members)):
            member = members[m]
            sp_dict[member] = sp_array_mean[m + 1, :]
            sp_dict[member + "_rot"] = sp_array_mean_vor[m + 1, :]
            sp_dict[member + "_div"] = sp_array_mean_div[m + 1, :]
    else:
        sp_dict = {}
        obs_name = para["ob_data"]["data_name"]
        sp_dict[obs_name] = sp_array_mean[0, :]
        for m in range(len(members)):
            member = members[m]
            sp_dict[member] = sp_array_mean[m + 1, :]

    return sp_dict


def plot_kinetic_energy_spectrum(sp_dict,  para,
                                 target_members=None,
                                 save_path=None, title=None, show=None):
    """
    绘制动能谱图
    参数:
    sp_dict: 包含各成员谱数据的字典
    grid0: 网格信息
    para: 参数字典
    target_members: 要绘制的成员列表，None表示绘制所有成员
    save_path: 图片保存路径
    title: 图片标题
    show: 是否显示图形
    """
    ns = 2

    grid0 = para["grid"]
    # 获取观测名称
    obs_name = para["ob_data"].get("data_name", "obs")

    # 确定要绘制的键
    if target_members is None:
        keys = list(sp_dict.keys())
    else:
        keys = []
        for member in target_members:
            # 添加成员的所有变体（原始、旋转、辐散）
            if member in sp_dict:
                keys.append(member)
            if f"{member}_rot" in sp_dict:
                keys.append(f"{member}_rot")
            if f"{member}_div" in sp_dict:
                keys.append(f"{member}_div")

    # 定义颜色列表
    colors = ['#ff7f0e', '#1f77b4', '#2ca02c', '#d62728', '#9467bd',
              '#8c564b', '#e377c2', '#7f7f7f', '#bcbd22', '#17becf']

    # 计算所有谱数据的最小值
    vmin = min(np.min(sp_dict[key][ns:]) for key in sp_dict.keys())

    # 创建图形
    plt.figure(figsize=(10, 6))
    color_index = 0
    color_dict = {}

    for key in keys:
        if key not in sp_dict:
            continue  # 跳过不存在的数据
        sp = sp_dict[key][ns:]
        # 确定基名和颜色
        base_key = key[:-4] if key.endswith(('_rot', '_div')) else key
        if base_key not in color_dict:
            if base_key == obs_name:
                color_dict[base_key] = 'black'
            else:
                color_dict[base_key] = colors[color_index % len(colors)]
                color_index += 1
        color = color_dict[base_key]

        # 确定线型
        if key.endswith('_rot'):
            linestyle = '--'
        elif key.endswith('_div'):
            linestyle = '-.'
        else:
            linestyle = '-'

        plt.plot(sp, linestyle, label=key, linewidth=1.2, color=color)

    # 添加参考线
    x = np.arange(10, 100, 1.0)

    # -3次方定律
    y = np.power(x, -3.0)
    y = 10 * y / np.max(y)
    plt.plot(x, y, "-.", label="-3 power law", linewidth=1.0, color="gray", alpha=0.7)

    # -5/3次方定律
    y = np.power(x, -5 / 3.0)
    y = 10 * y / np.max(y)
    plt.plot(x, y, "--", label="-5/3 power law", linewidth=1.0, color="gray", alpha=0.7)

    # 添加垂直参考线
    for vline in para["vline"]:
        plt.axvline(x=int(grid0.nlon / vline), color='cyan',
                    linestyle='--', linewidth=0.8, alpha=0.7,
                    label=f'{vline}$\\bigtriangleup$')

    plt.yscale('log')
    plt.xscale('log')
    plt.xlabel('WaveNumber')
    plt.ylim(vmin / 3, 2e2)

    # 处理标题
    if title is None:
        if target_members is None:
            title = "Kinetic Energy Spectrum - All Members"
        else:
            title = f"Kinetic Energy Spectrum - {', '.join(target_members)}"
    plt.title(title, fontsize=14)

    plt.ylabel('Kinetic energy($m^2s^{-2}$)', fontsize=12)
    plt.legend(loc="best", fontsize=9, ncol=2)
    plt.grid(True, which="both", ls="--", alpha=0.3)

    # 处理保存路径
    if save_path is not None:
        print(f"Saving figure to: {save_path}")
        plt.savefig(save_path, dpi=600, bbox_inches='tight')

    if show:
        plt.show()
    plt.close()

if __name__ == "__main__":
    para = {
        "grid": meteva.base.grid([0,359.75,0.25],[-89.875,89.875,0.25]),  # 检验区域
        "begin_time": datetime.datetime(2018, 9, 1, 0),  # 时段开始时刻(基于起报时间)
        "end_time": datetime.datetime(2018, 9, 2, 0),  # 时段结束时刻（基于起报时间）
        "time_step": 12,  # 起报时间间隔
        "dtime": 12,  # 预报时效
        "time_type": "BT",  # 最终检验结果呈现时，采用北京时还是世界时，UT代表世界时，BT代表北京时
        "need_Rotational_Divergent":True,
        "vline":[5,10],
        "ob_data": {
            "data_name":"ERA5",
            "u":{
            "dir_ob": r"\\10.28.16.234\data2\AI\Fengwu_ERA5\U\200\YYYYMMDDHH\YYYYMMDDHH.012.nc",  # 实况场数据路径
            "hour": None,
            "read_method": meb.read_griddata_from_nc,  # 读取数据的函数
            "operation": None,  # 预报数据读取后处理函数
            "operation_para": {},  # 预报数据读取后处理参数，用于对单位进行变换的操作
            "read_para": {},  # 读取数据的函数参数
            "time_type": "BT",  # 数据文件中的时间类型，UT代表世界时
            },
            "v":{
                "dir_ob": r"\\10.28.16.234\data2\AI\Fengwu_ERA5\V\200\YYYYMMDDHH\YYYYMMDDHH.012.nc",  # 实况场数据路径
                "hour": None,
                "read_method": meb.read_griddata_from_nc,  # 读取数据的函数
                "operation": None,  # 预报数据读取后处理函数
                "operation_para": {},  # 预报数据读取后处理参数，用于对单位进行变换的操作
                "read_para": {},  # 读取数据的函数参数
                "time_type": "BT",  # 数据文件中的时间类型，UT代表世界时
            }
        },
        "fo_data": {
            "FENGWU": {
                "u":{
                    "dir_fo": r"\\10.28.16.234\data2\AI\Fengwu_ERA5\U\200\YYYYMMDDHH\YYYYMMDDHH.TTT.nc",  # 数据路径
                    "read_method": meb.read_griddata_from_nc,  # 读取数据的函数
                    "read_para": {},  # 读取数据的函数参数
                    "reasonable_value": [0, 1000],  # 合理的预报值的取值范围，超出范围观测将被过滤掉
                    "operation": None,  # 预报数据读取后处理函数，用于对单位进行变换的操作
                    "operation_para": {},  # #预报数据读取后处理参数
                    "time_type": "BT",  # 预报数据时间类型是北京时，即08时起报
                    "move_fo_time": 0  # 是否对预报的时效进行平移，12 表示将1月1日08时的36小时预报转换成1月1日20时的24小时预报后参与对比
                },
                "v":{
                    "dir_fo": r"\\10.28.16.234\data2\AI\Fengwu_ERA5\V\200\YYYYMMDDHH\YYYYMMDDHH.TTT.nc",  # 数据路径
                    "read_method": meb.read_griddata_from_nc,  # 读取数据的函数
                    "read_para": {},  # 读取数据的函数参数
                    "reasonable_value": [0, 1000],  # 合理的预报值的取值范围，超出范围观测将被过滤掉
                    "operation": None,  # 预报数据读取后处理函数，用于对单位进行变换的操作
                    "operation_para": {},  # #预报数据读取后处理参数
                    "time_type": "BT",  # 预报数据时间类型是北京时，即08时起报
                    "move_fo_time": 0  # 是否对预报的时效进行平移，12 表示将1月1日08时的36小时预报转换成1月1日20时的24小时预报后参与对比
                }
            },
            "FUXI": {
                "u": {
                    "dir_fo": r"\\10.28.16.234\data2\AI\fuxi_ERA5\U\200\YYYYMMDDHH\YYYYMMDDHH.TTT.nc",  # 数据路径
                    "read_method": meb.read_griddata_from_nc,  # 读取数据的函数
                    "read_para": {},  # 读取数据的函数参数
                    "reasonable_value": [0, 1000],  # 合理的预报值的取值范围，超出范围观测将被过滤掉
                    "operation": None,  # 预报数据读取后处理函数，用于对单位进行变换的操作
                    "operation_para": {},  # #预报数据读取后处理参数
                    "time_type": "BT",  # 预报数据时间类型是北京时，即08时起报
                    "move_fo_time": 0  # 是否对预报的时效进行平移，12 表示将1月1日08时的36小时预报转换成1月1日20时的24小时预报后参与对比
                },
                "v": {
                    "dir_fo": r"\\10.28.16.234\data2\AI\fuxi_ERA5\V\200\YYYYMMDDHH\YYYYMMDDHH.TTT.nc",  # 数据路径
                    "read_method": meb.read_griddata_from_nc,  # 读取数据的函数
                    "read_para": {},  # 读取数据的函数参数
                    "reasonable_value": [0, 1000],  # 合理的预报值的取值范围，超出范围观测将被过滤掉
                    "operation": None,  # 预报数据读取后处理函数，用于对单位进行变换的操作
                    "operation_para": {},  # #预报数据读取后处理参数
                    "time_type": "BT",  # 预报数据时间类型是北京时，即08时起报
                    "move_fo_time": 0  # 是否对预报的时效进行平移，12 表示将1月1日08时的36小时预报转换成1月1日20时的24小时预报后参与对比
                }
            },
        },
        "output_dir": None  # 观测站点合并数据的输出路径，设置为None时不输出收集数据的中间结果
    }

    # 1. 计算动能谱
    sp_dict = kinetic_energy_spectrum(para)

    # 2. 绘制各种图表
    # 2.1 仅绘制观测
    plot_kinetic_energy_spectrum(
        sp_dict=sp_dict,
        para=para,
        target_members=["ERA5"],
        save_path="kes_obs.png",
        title="Kinetic Energy Spectrum - Observations"
    )

    # 2.2 仅绘制单个试验组
    plot_kinetic_energy_spectrum(
        sp_dict=sp_dict,
        para=para,
        target_members=["FENGWU"],
        save_path="kes_single_CMA_GFS.png",
        title="Kinetic Energy Spectrum - CMA_GFS"
    )

    # 2.3 绘制多个试验组对比
    plot_kinetic_energy_spectrum(
        sp_dict=sp_dict,
        para=para,
        target_members=["ERA5", "FENGWU"],
        save_path="kes_comparison_ERA5_CMA_GFS.png",
        title="Kinetic Energy Spectrum - ERA5 vs CMA_GFS"
    )

    # 2.4 所有成员在一张图
    plot_kinetic_energy_spectrum(
        sp_dict=sp_dict,
        para=para,
        target_members=None,  # None表示绘制所有成员
        save_path="kes_all.png",
        title="Kinetic Energy Spectrum - All Members"
    )

    # 2.5 绘制旋转分量和辐散分量
    plot_kinetic_energy_spectrum(
        sp_dict=sp_dict,
        para=para,
        target_members=["ERA5_rot", "ERA5_div", "FENGWU_rot", "FENGWU_div"],
        save_path="kes_rot_div.png",
        title="Kinetic Energy Spectrum - Rotational and Divergent Components"
    )


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Nov 12 17:48:39 2023

@author: tangbuxing
"""
import copy

#import pyreadr
import numpy as np
import pandas as pd
#import statsmodels.api as sm
from sklearn.preprocessing import Binarizer
import meteva.base
from  meteva.method.space.gbeta import Gbeta_score
import matplotlib.pyplot as plt

from meteva.method.space.qq_plot import get_qqplot_2samples_data

def ubalancer(x, alpha, beta):
    res = np.maximum(1 - (x - alpha)/(beta - alpha), 0)    #pmax -> np.maximum(np.arange(1,6), 3)
    return res

def G2IL_score(ob, fo, threshold, beta = None, alpha = 0):

    out = {}
    binarizer=Binarizer(threshold=threshold)
    Z = binarizer.transform(ob)
    Zhat = binarizer.transform(fo)
    G = Gbeta_score(Z,  Zhat, threshold = 0.5, beta = beta, alpha = alpha)
    
    IA = Z == 1
    IB = Zhat == 1
    nA = np.sum(IA)
    nB = np.sum(IB)
    nAB = np.sum(IA & IB)
    N = np.prod(ob.shape) +0.0
    if beta is None:
        beta = N*N  #/2.0
    
    if nA > 0 and nB > 0:
        x_data, y_data = get_qqplot_2samples_data(ob[IA], fo[IB], count=100)
        #tmp = sm.qqplot_2samples(X[IA],X[IB],line='45')    #sm.qqplot_2samples 未测
        #mae = np.mean(abs(tmp[x] - tmp[y]))
        #rmse = np.sqrt(np.mean((tmp[x] - tmp[y]), 2))
        
        #fig = sm.qqplot_2samples(ob[IA],ob[IB])    #sm.qqplot_2samples 未测
        #xx_qqplot = np.sort(X[IB])
        #yy_qqplot = np.sort(X[IA])
        # 获取 Q-Q 图中的数据
        #ax = fig.axes[0]
        #line = ax.get_lines()[0]  # 获取第一条线，即对角线
        #x_data, y_data = line.get_data()
        #rho = np.corrcoef(x_data, y_data)[0, 1]
        mae = np.mean(abs(x_data - y_data))
        rmse = np.sqrt(np.mean(np.power((x_data - y_data),2)))
        #print(mae,rmse)
            
    elif nA == nB:
        mae = rmse = 0
    elif nA > 0 :
        mae = rmse = np.max(abs(ob))
    elif nB > 0 :
        mae = rmse = np.max(abs(fo))
    else:
        mae = meteva.base.IV
        rmse = meteva.base.IV
    #x = (nA + nB - 2 * nAB) * (1 + mae)    
    #out = np.maximum(1 - (x - alpha)/(beta - alpha), 0)    #pmax -> np.maximum(np.arange(1,6), 3)

    out_G2IL = ubalancer(x = (nA + nB - 2 * nAB) * (1 + mae), alpha = alpha, 
                   beta = beta)  
  
    comps = copy.deepcopy(G)
    comps["rmse_sorted"] = rmse
    comps['G2IL']= out_G2IL
    
    return comps
    
if __name__ == '__main__':
    # fcst = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/Gbeta/data/wrf4ncar0531.RData')['wrf4ncar0531']
    # obs = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/Gbeta/data/obs0601.RData')['obs0601']
    # res_G2IL = G2IL(X = obs.values, Xhat = fcst.values, threshold = 2.1,beta = 601 * 501 )

    import meteva.base as meb
    grid1 = meb.grid([100, 120, 0.05], [24, 40, 0.05])
    path_ob = r'H:\test_data\input\mem\mode\ob\rain03\20072611.000.nc'
    path_fo_03 = r'H:\test_data\input\mem\mode\ec\rain03\20072608.003.nc'
    grd_ob = meb.read_griddata_from_nc(path_ob, grid=grid1, time="2020072611", dtime=0, data_name="OBS")
    grd_fo = meb.read_griddata_from_nc(path_fo_03, grid=grid1, time="2020072608", dtime=3, data_name="ECMWF")
    res_Gbeta = G2IL_score(grd_ob.values.squeeze(), grd_fo.values.squeeze(), threshold = 0.1)
    print(res_Gbeta)

'''
    X = fcst.values
    Xhat = obs.values
    threshold = 2.1
    alpha = 0
    rule = ">"
    w = 0.5
'''

    
    
    


import os.path

import meteva.base as meb
import datetime
import numpy as np
import matplotlib.pyplot as plt
import meteva
import matplotlib.ticker as ticker

def spectrum_fft2_one_field(grd):
    data = grd.values.squeeze()
    fft_result = np.fft.fft2(data)
    # 移位使得零频率在中心
    fft_shifted = np.fft.fftshift(fft_result)
    # 计算能量谱
    energy_spectrum = np.abs(fft_shifted) ** 2

    # 获取二维能谱的尺寸
    ny, nx = energy_spectrum.shape

    # 创建一个与能谱尺寸相同的坐标网格
    x = np.arange(nx) - nx // 2
    y = np.arange(ny) - ny // 2
    xx, yy = np.meshgrid(x, y)

    # 计算每个点到中心的距离
    r = np.sqrt(xx ** 2 + yy ** 2)

    # 定义距离的离散区间
    nbins = int(np.sqrt(nx ** 2 + ny ** 2))
    bins = np.arange(nbins)

    # 进行径向平均
    hist, bin_edges = np.histogram(r, bins=bins, weights=energy_spectrum)
    counts, _ = np.histogram(r, bins=bins)
    radial_average = hist / counts
    mask = np.logical_not(np.isnan(radial_average))
    radial_average = radial_average[mask]

    return radial_average

def spectrum_fft2(para,show = None,save_path = None,title = None):
    grid0 = para["grid"]
    begin_time = para["begin_time"]
    end_time = para["end_time"]
    dtime = para["dtime"]
    time1 = begin_time - datetime.timedelta(hours=dtime)
    members = list(para["fo_data"].keys())

    sp_list_list = []
    while time1 <= end_time:
        time1 += datetime.timedelta(hours=dtime)

        time_ob = time1 + datetime.timedelta(hours=dtime)
        if para["time_type"] != para["ob_data"]["time_type"]:
            if para["time_type"]=="UT":
                time_ob_file = time_ob + datetime.timedelta(hours=8)
            else:
                time_ob_file = time_ob - datetime.timedelta(hours=8)
        else:
            time_ob_file = time_ob

        path_ob = meb.get_path(para["ob_data"]["dir_ob"],time_ob_file)
        if not os.path.exists(path_ob):continue
        grd = para["ob_data"]["read_method"](path_ob,grid = grid0,**para["ob_data"]["read_para"],show = True)
        if grd is None:continue
        if para["ob_data"]["operation"] is not None:
            grd = para["ob_data"]["operation"](grd,**para["ob_data"]["operation_para"])

        sp_list = []
        sp = spectrum_fft2_one_field(grd)
        sp_list.append(sp)

        for member in members:

            if para["time_type"] != para["fo_data"][member]["time_type"]:
                if para["time_type"] == "UT":
                    time_fo_file = time1 + datetime.timedelta(hours=8)
                else:
                    time_fo_file = time1 - datetime.timedelta(hours=8)
            else:
                time_fo_file = time1

            path_fo = meb.get_path(para["fo_data"][member]["dir_fo"], time_fo_file,dtime)
            if not os.path.exists(path_fo): continue
            grd = para["fo_data"][member]["read_method"](path_fo, grid=grid0, **para["fo_data"][member]["read_para"],show = True)
            if grd is None: continue
            if para["fo_data"][member]["operation"] is not None:
                grd = para["fo_data"][member]["operation"](grd, **para["fo_data"][member]["operation_para"])


            sp = spectrum_fft2_one_field(grd)
            sp_list.append(sp)

        if len(sp_list) == len(para["fo_data"].keys()) +1:
            sp_list_list.append(sp_list)


    sp_array = np.array(sp_list_list)
    sp_array_mean = np.mean(sp_array,axis=0)  #时间维度取平均
    sp_dict = {}
    sp_dict["obs"] = sp_array_mean[0,:]
    for m in range(len(members)):
        member = members[m]
        sp_dict[member] = sp_array_mean[m+1,:]


    if show is not None or save_path is not None:
        ns = 2
        ne = int(grid0.nlon / 4)
        for key in sp_dict.keys():
            sp = sp_dict[key]
            plt.plot(sp[ns:ne], label=key, linewidth=1)

        plt.yscale('log')  # 设置Y轴为对数坐标
        plt.xscale('log')  # 设置Y轴为对数坐标
        # 设置更多的刻度
        ticks = [1,2,3,4,5,7,10,20,30,40,60,100]
        labels = [str(t) for t in ticks]
        plt.xticks(ticks, labels)

        plt.xlabel('波数')

        if title is not None:
            plt.title(title)
        plt.ylabel('Power')
        plt.legend()

        if save_path is not None:
            plt.savefig(save_path, dpi=600)
            print("png result has been output to " + save_path)
        if show is not None:
            plt.show()

        plt.close()

    return sp_dict


if __name__ == "__main__":
    para = {
        "grid": meteva.base.grid([0,359.75,0.25],[-89.875,89.875,0.25]),  # 检验区域
        "begin_time": datetime.datetime(2018, 9, 1, 0),  # 时段开始时刻(基于起报时间)
        "end_time": datetime.datetime(2018, 9, 1, 0),  # 时段结束时刻（基于起报时间）
        "time_step": 12,  # 起报时间间隔
        "dtime": 12,  # 预报时效
        "time_type": "BT",  # 最终检验结果呈现时，采用北京时还是世界时，UT代表世界时，BT代表北京时
        "ob_data": {
            "dir_ob": r"\\10.28.16.234\data2\AI\Fengwu_ERA5\U\200\YYYYMMDDHH\YYYYMMDDHH.012.nc",  # 实况场数据路径
            "hour": None,
            "read_method": meb.read_griddata_from_nc,  # 读取数据的函数
            "operation": None,  # 预报数据读取后处理函数
            "operation_para": {},  # 预报数据读取后处理参数，用于对单位进行变换的操作
            "read_para": {},  # 读取数据的函数参数
            "time_type": "BT",  # 数据文件中的时间类型，UT代表世界时
        },
        "fo_data": {
            "CMA_GFS": {
                "dir_fo": r"\\10.28.16.234\data2\AI\Fengwu_ERA5\U\200\YYYYMMDDHH\YYYYMMDDHH.TTT.nc",  # 数据路径
                "read_method": meb.read_griddata_from_nc,  # 读取数据的函数
                "read_para": {},  # 读取数据的函数参数
                "reasonable_value": [0, 1000],  # 合理的预报值的取值范围，超出范围观测将被过滤掉
                "operation": None,  # 预报数据读取后处理函数，用于对单位进行变换的操作
                "operation_para": {},  # #预报数据读取后处理参数
                "time_type": "BT",  # 预报数据时间类型是北京时，即08时起报
                "move_fo_time": 0  # 是否对预报的时效进行平移，12 表示将1月1日08时的36小时预报转换成1月1日20时的24小时预报后参与对比
            },
        },
        "output_dir": None  # 观测站点合并数据的输出路径，设置为None时不输出收集数据的中间结果
    }

    spectrum_fft2(para,save_path=r"H:\test_data\output\method\space\spectrum\test.png")


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jun 12 14:05:22 2023

@author: tangbuxing
"""
import copy

#import pyreadr
import numpy as np
import pandas as pd

import meteva.base
#import statsmodels.api as sm
from  meteva.method.space.gbeta import Gbeta_score
import matplotlib.pyplot as plt

from meteva.method.space.qq_plot import get_qqplot_2samples_data
from sklearn.preprocessing import Binarizer

def GbetaIL_score(ob, fo, threshold, beta= None, alpha = 0,  w = 0.5):


    out = {}
    binarizer=Binarizer(threshold=threshold)
    Z = binarizer.transform(ob)
    Zhat = binarizer.transform(fo)
    G = Gbeta_score(Z, Zhat, threshold = 0.5, beta = beta, alpha = alpha)
    
    IA = Z == 1
    IB = Zhat == 1
    nA = np.sum(IA)
    nB = np.sum(IB)
    nAB = np.sum(IA & IB)
    N = np.prod(ob.shape) +0.0
    if beta is None:
        beta = N*N  #/2.0


    if nA > 0 and nB > 0:
        x_data,y_data = get_qqplot_2samples_data(ob[IA],fo[IB],count=100)
        #fig = sm.qqplot_2samples(X[IA],X[IB])    #sm.qqplot_2samples 未测
        #xx_qqplot = np.sort(ob[IB])
        #yy_qqplot = np.sort(fo[IA])
        # 获取 Q-Q 图中的数据
        #ax = fig.axes[0]
        #line = ax.get_lines()[0]  # 获取第一条线，即对角线
        #x_data, y_data = line.get_data()
        #cor计算相关性系数 —>np.corrcoef(data)[0,1]

        rho = np.corrcoef(x_data, y_data)[0, 1]
        
    elif nA == nB:
        rho = 1
    elif nA > 0 :
        rho = 1 - (nA/N)
    elif nB > 0 :
        rho = 1 - (nB/N)
    else:
        rho = meteva.base.IV

    rho = max(rho, 0)
    #out = w * G + (1 - w) * rho
    out_GbetaIL = w * 0 + (1 - w) * rho    #G为Gbeta(A,B)的值，暂不明确Gbeta函数中从哪得到
    comps = copy.deepcopy(G)
    comps['corrcoef_sorted'] = rho
    
    comps['GbetaIL']= out_GbetaIL
    comps["weights"]=[w, 1 - w]
    
    return comps
    

if __name__ == '__main__':
    #fcst = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/Gbeta/data/wrf4ncar0531.RData')['wrf4ncar0531']
    #obs = pyreadr.read_r(r'/Users/tangbuxing/Desktop/tra_test-SAL/Gbeta/data/obs0601.RData')['obs0601']

    #res_GbetaIL = GbetaIL( X = obs.values, Xhat = fcst.values, threshold = 2.1,beta = 601 * 501 )

    import meteva.base as meb
    grid1 = meb.grid([100, 120, 0.05], [24, 40, 0.05])
    path_ob = r'H:\test_data\input\mem\mode\ob\rain03\20072611.000.nc'
    path_fo_03 = r'H:\test_data\input\mem\mode\ec\rain03\20072608.003.nc'
    grd_ob = meb.read_griddata_from_nc(path_ob, grid=grid1, time="2020072611", dtime=0, data_name="OBS")
    grd_fo = meb.read_griddata_from_nc(path_fo_03, grid=grid1, time="2020072608", dtime=3, data_name="ECMWF")
    res_Gbeta = GbetaIL_score( grd_ob.values.squeeze(), grd_fo.values.squeeze(), threshold = 0.1)
    print(res_Gbeta)



    